using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Globalization;
using Microsoft.Win32;
using System.Threading;
using System.Reflection;
using System.IO;
using WillStott.MxAppToolbox;
using osImageManagerLib;


namespace osImageManagerApp
{
    /// <summary>
    /// Summary description for Main.
    /// </summary>
    public class osImageManagerProgram : Program
    {
        private bool _mainFormClosing;
        private UniMsgList _uniMsgList;
        private UniLog _uniEventLog;
        private string _helpFilePathName;
        private string _tmpFilePathName;

        private bool _verboseLog;		//registry values
        private Int32 _runCount;
        private string _userName;
        private string _lastFileImportDir;
        private string _lastFileExportDir;
        private bool _surnameFirstQueryUKCTOCS;
        private bool _autoReadAfterBrowseFileImport;
        private string _examDirectoryNamePattern;
        private DlgProcFileImport.ImportMode _importMode;


        private osImageManagerProgram() { }
        static osImageManagerProgram()
        {
            try
            {
                Program.Instance = new osImageManagerProgram();

                osImageManagerProgram.Instance._mainFormClosing = true;	//reset in MainFrom when it is open

                osImageManagerProgram.Instance._uniEventLog = new UniLog();
                osImageManagerProgram.Instance._uniEventLog.Name = "osImageManagerApp";
                osImageManagerProgram.Instance._verboseLog = true;
                osImageManagerProgram.Instance._tmpFilePathName = Path.GetTempFileName();
                osImageManagerProgram.Instance._importMode = DlgProcFileImport.ImportMode.AllFilesInTree;
                osImageManagerProgram.Instance._examDirectoryNamePattern = ExaminationSet.DirRegexPattern;

                osImageManagerProgram.Instance.ReadRegistryValues();
                osImageManagerProgram.Instance._runCount++;
                osImageManagerProgram.Instance._helpFilePathName = ExecutionTime.Instance.InstallDir + "osImageManagerApp.chm";

                osImageManagerProgram.Instance.Log("Application started - Run number {0}", osImageManagerProgram.Instance.RunCount);
            }
            catch (Exception e)
            {		//unless you end the program here a TypeInitializationException is thrown as the ctor failed
                Program.FatalErrorAbend("osImageManagerApp.osImageManagerApp()", e.Message);
            }
        }
        static public new osImageManagerProgram Instance
        {
            get { return Program.Instance as osImageManagerProgram; }
        }

        public override void MainArgProc(string[] args)
        {
            Assembly resAsm = Program.GetResourceAssembly(this);
            osImageManagerProgram.Instance._uniMsgList = UniMsgList.LoadList(resAsm, "osImageManagerApp.ImageMngrMsg.xml");
        }

        public override string HelpFilePathName
        {
            get { return _helpFilePathName; }
            set { _helpFilePathName = value; }
        }

        public override UniLog AppLog
        {
            get { return _uniEventLog; }
        }
        public override bool VerboseLog
        {
            get { return _verboseLog; }
            set { _verboseLog = value; }
        }

        public override bool MainFormClosing
        {
            get { return _mainFormClosing; }
            set { _mainFormClosing = value; }
        }

        public override UniMsgList ErrMsgs
        {
            get { return _uniMsgList; }
        }

        public override bool CloseAllDocuments(bool AllowCancel)
        {
            bool rc = true;

            if (AllowCancel == false)
                osImageManagerProgram.Instance.Log("Application forcibly closed");
            else
            {
               osImageManagerProgram.Instance.Log("Application closed normally");
            }
            if (rc == true)
                WriteRegistryValues(true);
            return rc;
        }

        public DlgProcFileImport.ImportMode ImageImportMode
        {
            get { return _importMode; }
            set { _importMode = value; }
        }
        public string ExamDirectoryNamePattern
        {
            get { return _examDirectoryNamePattern; }
            set { _examDirectoryNamePattern = value; }
        }

        public string TempDicomPathFile
        {
            get { return _tmpFilePathName; }
        }

        public string LastFileImportDir
        {
            get { return _lastFileImportDir; }
            set { _lastFileImportDir = value; }
        }

        public string LastFileExportDir
        {
            get { return _lastFileExportDir; }
            set { _lastFileExportDir = value; }
        }

        public string UserName
        {
            get { return _userName; }
            set { _userName = value; }
        }

        public int RunCount
        {
            get { return (int)_runCount; }
        }
        public bool SurnameFirstQueryUKCTOCS
        {
            get { return _surnameFirstQueryUKCTOCS; }
            set { _surnameFirstQueryUKCTOCS = value; }
        }
        public bool AutoReadAfterBrowseFileImport
        {
            get { return _autoReadAfterBrowseFileImport; }
            set { _autoReadAfterBrowseFileImport = value; }
        }

        private void ReadRegistryValues()
        {
            using (RegistryKey root = Registry.CurrentUser)
            {
                try
                {
                    RegistryKey appKey = root.CreateSubKey(@"osImageManagerApp");

                    _runCount = (Int32)appKey.GetValue("RunCount", 0);
                    _userName = appKey.GetValue("LastUserName", "") as string;
                    if (_userName.Length == 0)		//if default string is used, set m_Author to null
                        _userName = null;
                    _verboseLog = ((Int32)appKey.GetValue("VerboseLog", (_verboseLog == true) ? 1 : 0) == 1) ? true : false;
                    _lastFileImportDir = appKey.GetValue("LastFileImportDir", "c:\\") as string;
                    _lastFileExportDir = appKey.GetValue("LastFileExportDir", "c:\\") as string;
                    _surnameFirstQueryUKCTOCS = ((Int32)appKey.GetValue("SurnameFirst", (_surnameFirstQueryUKCTOCS == true) ? 1 : 0) == 1) ? true : false;
                    _autoReadAfterBrowseFileImport = ((Int32)appKey.GetValue("AutoReadAfterBrowse", (_autoReadAfterBrowseFileImport == true) ? 1 : 0) == 1) ? true : false;
                    _importMode = DlgProcFileImport.Int32ToImportMode((Int32)appKey.GetValue("ImportMode", 0));
                    _examDirectoryNamePattern = appKey.GetValue("ExamDirNamePattern", ExaminationSet.DirRegexPattern) as string;
                }
                catch (Exception)
                {
                }
            }
        }

        public void WriteRegistryValues(bool UserSettable)
        {
            using (RegistryKey root = Registry.CurrentUser)
            {
                try
                {
                    RegistryKey appKey = root.CreateSubKey(@"osImageManagerApp");

                    appKey.SetValue("RunCount", _runCount);
                    if (UserSettable)
                    {
                        appKey.SetValue("LastUserName", (_userName == null) ? "" : _userName);
                        appKey.SetValue("VerboseLog", (_verboseLog == true) ? 1 : 0);
                        appKey.SetValue("LastFileImportDir", (_lastFileImportDir == null) ? "" : _lastFileImportDir);
                        appKey.SetValue("LastFileExportDir", (_lastFileExportDir == null) ? "" : _lastFileExportDir);
                        appKey.SetValue("SurnameFirst", (_surnameFirstQueryUKCTOCS == true) ? 1 : 0);
                        appKey.SetValue("AutoReadAfterBrowse", (_autoReadAfterBrowseFileImport == true) ? 1 : 0);
                        appKey.SetValue("ImportMode", DlgProcFileImport.ImportModeToInt32(_importMode));
                        appKey.SetValue("ExamDirNamePattern", (_examDirectoryNamePattern == null) ? "" : _examDirectoryNamePattern);
                    }
                }
                catch (Exception)
                {
                }
            }
        }

        public static void DeleteTempFile()
        {
            try
            {
                if (osImageManagerProgram.Instance._tmpFilePathName != null)
                    File.Delete(osImageManagerProgram.Instance._tmpFilePathName);
                osImageManagerProgram.Instance._tmpFilePathName = null;
            }
            catch (Exception)
            {
            }
        }


        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            osImageManagerProgram.Instance.MainArgProc(args);

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new MainForm());

            osImageManagerProgram.DeleteTempFile();
        }
    }
}