using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DicomFile
    {
        private ImageRecord _record;            //filename is stored in _record.ImagePathFilename
        private bool _DicomHeader;       //true if identifier found in header of associated file
        private DicomMask _mask;
        private int _rowsImage;                 //number of rows in image data of associated file (0028,0010)
        private int _colsImage;                 //number of cols in image data of associated file (0028,0011)

        public static readonly string DateFormat = "yyyyMMdd";
        public static readonly string StdExt = ".dicom";
        private static bool UATFileGeneration = false;        //set this mode ONLY when generating UAT Test Files - set DlgToolsDbAdmin.cs
                                                
        private enum ProcessCacheAction
        {
            UpdateCacheFromRecordHeaderFields = 1,
            RestoreMaskedDataToCache = 2,                  
            UpdateRecordHeaderFieldsFromCache = 3, 
        }

        public DicomFile()
        {
            _record = new ImageRecord();
            _DicomHeader = false;
            _mask = new DicomMask();
        }

        public DicomFile(string pathFilename)
        {
            _record = new ImageRecord(pathFilename);
            _DicomHeader = false;
            _mask = new DicomMask();
        }

        public DicomFile(ImageRecord record)
        {
            if (record == null)
                throw new UniException("OSA-053", UniException.Reset.TermApp, UniException.Category.Program, "DicomFile ctor passed null DicomRecord");
            _record = record;
            _DicomHeader = _record.HeaderFieldsInitiated();
            _mask = new DicomMask();
        }

        public override string ToString()
        {
            string rc = "[not initialized]";
            if (_record != null)
                rc = _record.ToString();
            return rc;
        }

        public ImageRecord Record { get { return _record; } }
        public bool IsDicomHeader() { return _DicomHeader; }

        public string CreateDicomFilename()
        {
            return _record.CreateFilename() + DicomFile.StdExt;
        }

        public static bool IsValidFilename(FileSystemInfo info)
        {
            bool rc = false;
            if (info != null)
            {
                if (info.Extension.Length == 0)
                    rc = true;
                else if (String.Compare(info.Extension, DicomFile.StdExt, true) == 0)
                    rc = true;
            }
            return rc;
        }

        public bool WriteToDisk(OsDatabases db, bool RestoreMaskedData, string pathFilename) //, Program progErrorHandler)
        {
            bool rc = false;

            if ((_record == null) || (db == null) || (pathFilename == null) )
                throw new UniException("OSA-051", UniException.Reset.AbortOp, UniException.Category.System, "_record, db or pathFilename is null");

            if (_record.IsDicomFileCached() == false)
                db.LoadImageFileDataForImageID(_record);

            if (_record.IsDicomFileCached() == true)
            {
                StreamWriter writer = null;
                BinaryWriter binwriter = null;

                try
                {
                    writer = new StreamWriter(pathFilename);
                    binwriter = new BinaryWriter(writer.BaseStream);

                    if ( RestoreMaskedData )
                        ProcessCache(ProcessCacheAction.RestoreMaskedDataToCache);

                    binwriter.Write(_record.GetDicomFileCache());
                    binwriter.Flush();

                    if (RestoreMaskedData)
                        _record.ClearDicomFileCache();

                    rc = true;
                }
                catch (Exception ex)
                {
                    throw new UniException("OSA-050", UniException.Reset.AbortOp, UniException.Category.System, ex, "Problem writing DICOM file: {0}", pathFilename);
                }
                finally
                {
                    if (writer != null)
                        writer.Close();
                    if (binwriter != null)
                        binwriter.Close();
                }
            }
            return rc;
        }

        public void TestReadFromDisk(string filename)
        {
            try
            {
                using (StreamReader reader = new StreamReader(filename))
                {
                    using (BinaryReader binreader = new BinaryReader(reader.BaseStream))
                    {
                        FileInfo fileInfo = new FileInfo(filename);
                        if (fileInfo.Length > int.MaxValue)
                            throw new InvalidOperationException("file is too big");

                        if (_record.SetDicomFileCache(binreader.ReadBytes((int)fileInfo.Length)) == 0)
                            throw new InvalidOperationException("file cannot be read");
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
            return;
        }

        public bool ReadFromDisk(OsDatabases db, Program progErrorHandler)
        {
            bool rc = false;

            if (progErrorHandler == null)
                return false;
            try
            {
                using (StreamReader reader = new StreamReader(_record.DicomPathFilename))
                {
                    using (BinaryReader binreader = new BinaryReader(reader.BaseStream))
                    {
                        FileInfo fileInfo = new FileInfo(_record.DicomPathFilename);
                        if (fileInfo.Length > int.MaxValue)
                            progErrorHandler.Log("Error: DicomFile.ReadFromDisk() - DICOM file is too big: {0}: {1}", fileInfo.Length, _record.DicomPathFilename);
                        else
                        {
                            if (_record.SetDicomFileCache(binreader.ReadBytes((int)fileInfo.Length)) > 0)
                            {
                                if (ProcessCache(ProcessCacheAction.UpdateRecordHeaderFieldsFromCache))
                                {
                                    if (_record.HeaderFieldsInitiated() == false)
                                        _record.UnspecifiedErrorText = Resources.StrErrorBadDicomHeader;
                                    else
                                    {
                                        rc = ((_record.SetValidationFlags(db, progErrorHandler) == true) && (_record.UnspecifiedError == false));
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                progErrorHandler.Log("Error: DicomFile.ReadFromDisk() - Exception reading DICOM file: {0}: msg[{1}]", _record.DicomPathFilename, ex.Message);
                _record.UnspecifiedErrorText = _record.DicomPathFilename + ": " + ex.Message;
            }
            return rc;
        }

        public bool UpdateAndMaskCache(Program progErrorHandler)
        {
            bool rc = false;

            if (progErrorHandler == null)
                return false;
            try
            {
                ProcessCache(ProcessCacheAction.UpdateCacheFromRecordHeaderFields);

                if (_mask.WriteMaskToDisk(_record.GetShortDicomRef(), progErrorHandler) == false) //if DicomMask.MaskDir == "" then do nothing and return true
                    _record.UnspecifiedErrorText = Resources.StrFailedToWriteMask;

                if ((_record.UnspecifiedError == false) || (_DicomHeader == true))
                {
                    _record.MaskFileLabel = _mask.MaskFileLabel;
                    rc = true;
                }
            }
            catch (Exception ex)
            {
                progErrorHandler.Log("Error: DicomFile.ReadFile() - Exception reading DICOM file: {0}: msg[{1}]", _record.DicomPathFilename, ex.Message);
                _record.UnspecifiedErrorText = _record.DicomPathFilename + ": " + ex.Message;
            }
            return rc;
        }

        private bool ProcessCache(ProcessCacheAction action)
        {
            bool rc = false;

            ASCIIEncoding encode = new ASCIIEncoding();

            _DicomHeader = false;
            if (action == ProcessCacheAction.UpdateRecordHeaderFieldsFromCache)
                _record.ClearHeaderFields();

            try
            {
                using (MemoryStream memw = new MemoryStream())
                {
                    using (BinaryWriter binwriter = new BinaryWriter(memw))
                    {
                        using (MemoryStream memr = new MemoryStream(_record.GetDicomFileCache()))
                        {
                            using (BinaryReader binreader = new BinaryReader(memr))
                            {
                                byte[] preamble = DicomFile.ReadBytes(128, binreader, binwriter);
                                byte[] id = DicomFile.ReadBytes(4, binreader, binwriter);

                                string dicom = encode.GetString(id, 0, 4);

                                if (dicom == "DICM")
                                {
                                    _DicomHeader = true;
                                    ushort Group = 0;
                                    ushort Element = 0;

                                    while (!((Group == 0x7FE0) && (Element == 0x0010)))
                                    {
                                        Group = DicomFile.ReadUint16(0, binreader, binwriter);
                                        Element = DicomFile.ReadUint16(0, binreader, binwriter);

                                        if (action == ProcessCacheAction.UpdateCacheFromRecordHeaderFields)
                                            UpdateCacheFromRecordHeaderFields(binreader, binwriter, encode, Group, Element);
                                        if (action == ProcessCacheAction.RestoreMaskedDataToCache)
                                            RestoreMaskToCache(binreader, binwriter, Group, Element);
                                        if (action == ProcessCacheAction.UpdateRecordHeaderFieldsFromCache)
                                            UpdateRecordHeaderFieldsFromCache(binreader, binwriter, encode, Group, Element);
                                    }
                                }
                                if ((action == ProcessCacheAction.UpdateCacheFromRecordHeaderFields) || (action == ProcessCacheAction.RestoreMaskedDataToCache))
                                    _record.SetDicomFileCache(memw.ToArray());
                                rc = true;
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
            return rc;
        }

        private void RestoreMaskToCache(BinaryReader binreader, BinaryWriter binwriter, ushort Group, ushort Element)
        {
            byte[] value = GetDataElementValue(false, Group, Element, binreader, binwriter);

            if ((Group == 0x7FE0) && (Element == 0x0010))       //image data - for mask
                _mask.RestoreMask(value, _record.GetShortDicomRef());

            binwriter.Write(value);
        }

        private void UpdateCacheFromRecordHeaderFields(BinaryReader binreader, BinaryWriter binwriter, ASCIIEncoding encode, ushort Group, ushort Element)
        {
            byte[] value = GetDataElementValue(true, Group, Element, binreader, binwriter);

            if ((Group == 0x0008) && (Element == 0x0018))       //SOP Instance UID - globally unique reference for image
            {
                value = new byte[encode.GetByteCount(_record.DicomRef)];
                value = encode.GetBytes(_record.DicomRef);
            }
            if ((Group == 0x0008) && (Element == 0x0020))       //StudyDate
            {
                value = new byte[encode.GetByteCount(DicomFile.ConvertStudyDate(_record.AppointmentDate))];
                value = encode.GetBytes(DicomFile.ConvertStudyDate(_record.AppointmentDate));
            }
            if ((Group == 0x0010) && (Element == 0x0020))       //PatientID
            {
                value = new byte[encode.GetByteCount(_record.VolunteerRef)];
                value = encode.GetBytes(_record.VolunteerRef);
            }
            if ((Group == 0x0010) && (Element == 0x0010))       //PatientName
            {
                if (DicomFile.UATFileGeneration == false)
                {
                    value = new byte[DicomMask.MaskPatientNameCnt];
                    for (int y = 0; y < DicomMask.MaskPatientNameCnt; y++)
                        value[y] = DicomMask.MaskPatientNameChar;
                }
                else
                {
                    value = new byte[encode.GetByteCount(_record.PatientName)];
                    value = encode.GetBytes(_record.PatientName);
                }
            }
            if ((Group == 0x7FE0) && (Element == 0x0010))       //image data - for mask, does nothing if _mask._applyMask == false
            {
                if (DicomFile.UATFileGeneration == false)
                {
                    _mask.ApplyMask(value, _rowsImage, _colsImage);
                }
            }
            binwriter.Write(value);
        }

        private void UpdateRecordHeaderFieldsFromCache(BinaryReader binreader, BinaryWriter binwriter, ASCIIEncoding encode, ushort Group, ushort Element)
        {
            byte[] value = GetDataElementValue(false, Group, Element, binreader, binwriter);

            if ((Group == 0x0008) && (Element == 0x0018))       //SOP Instance UID - globally unique reference for image
            {
                if (value[value.Length - 1] == 0x00)                      //bug in the way SonoView writes this field, it appends a null to the string
                    _record.DicomRef = encode.GetString(value, 0, value.Length - 1);
                else
                    _record.DicomRef = encode.GetString(value, 0, value.Length);
            }
            if ((Group == 0x0008) && (Element == 0x0020))       //StudyDate
            {
                _record.AppointmentDate = DicomFile.ConvertStudyDate(encode.GetString(value, 0, value.Length));
            }
            if ((Group == 0x0010) && (Element == 0x0020))       //PatientID
            {
                _record.VolunteerRef = DicomFile.ConvertPatientID(encode.GetString(value, 0, value.Length));
            }
            if ((Group == 0x0010) && (Element == 0x0010))       //PatientName
            {
                _record.PatientName = encode.GetString(value, 0, value.Length);
            }
            if ((Group == 0x0028) && (Element == 0x0010))       //Rows in image
            {
                _rowsImage = BitConverter.ToInt16(value, 0);
            }
            if ((Group == 0x0028) && (Element == 0x0011))       //Cols in image
            {
                _colsImage = BitConverter.ToInt16(value, 0);
            }
            binwriter.Write(value);

            return;
        }

        private byte[] GetDataElementValue(bool CalcNewElementLengths, ushort Group, ushort Element, BinaryReader binreader, BinaryWriter binwriter)
        {
            byte[] value = null;
            int dataLength = 0;

            char[] vra = DicomFile.ReadChars(2, binreader, binwriter);

            string vr = new string(vra, 0, 2);
            if ((vr != "OB") && (vr != "OW") && (vr != "OF") && (vr != "SQ") && (vr != "UT") && (vr != "UN"))
            {
                if (CalcNewElementLengths)
                    dataLength = GetDataElementLength(true, Group, Element, binreader, binwriter);
                else
                    dataLength = ReadUint16(0, binreader, binwriter);
            }
            else
            {
                DicomFile.ReadChars(2, binreader, binwriter);
                if (CalcNewElementLengths)
                    dataLength = GetDataElementLength(false, Group, Element, binreader, binwriter);
                else
                    dataLength = (int)ReadUint32(0, binreader, binwriter);
            }
            value = new byte[dataLength];
            value = binreader.ReadBytes(dataLength);
            return value;
        }

        private int GetDataElementLength(bool Uint16, ushort Group, ushort Element, BinaryReader binreader, BinaryWriter binwriter)
        {
            int dataLength = -1;

            if ((Group == 0x0008) && (Element == 0x0018))       //SOP Instance UID - globally unique reference for image
            {
               //BF-33 _record.UpdateDicomRef();   //this image will be changed so it must have a new SOP InstanceUID (0008,0018)
                dataLength = (Uint16) ? ReadUint16((ushort)_record.DicomRef.Length, binreader, binwriter) : (int)ReadUint32((uint)_record.DicomRef.Length, binreader, binwriter);
            }
            if ((Group == 0x0008) && (Element == 0x0020))       //StudyDate
            {
                dataLength = (Uint16) ? ReadUint16((ushort)DicomFile.DateFormat.Length, binreader, binwriter) : (int)ReadUint32((uint)DicomFile.DateFormat.Length, binreader, binwriter);
            }
            if ((Group == 0x0010) && (Element == 0x0020))       //PatientID
            {
                dataLength = (Uint16) ? ReadUint16((ushort)_record.VolunteerRef.Length, binreader, binwriter) : (int)ReadUint32((uint)_record.VolunteerRef.Length, binreader, binwriter);
            }
            if ((Group == 0x0010) && (Element == 0x0010))       //PatientName
            {
                if (DicomFile.UATFileGeneration == false)
                {
                    dataLength = (Uint16) ? ReadUint16(DicomMask.MaskPatientNameCnt, binreader, binwriter) : (int)ReadUint32(DicomMask.MaskPatientNameCnt, binreader, binwriter);
                }
                else
                {
                    dataLength = (Uint16) ? ReadUint16((ushort)_record.PatientName.Length, binreader, binwriter) : (int)ReadUint32((uint)_record.PatientName.Length, binreader, binwriter);
                }
            }
            if (dataLength == -1)
                dataLength = (Uint16) ? ReadUint16(0, binreader, binwriter) : (int)ReadUint32(0, binreader, binwriter);

            return dataLength;
        }

        public static string ConvertPatientID(string VolunteerRef)
        {
            string rc = (VolunteerRef == null) ? "-1" : VolunteerRef.ToUpper();
            try
            {
                int offsetU = 0;
                if ((offsetU = rc.IndexOf('U')) > -1)  //BF-2: remove any U characters added to the end of the volunteer ref - a hack to resolve SonoView problem
                    rc = rc.Substring(0, offsetU);
                if (rc.Length == 7)                    //BF-2: correct for dicom files created before volunteer ref was extended to eight digits
                    rc = rc.Insert(2, "0");

                int val = int.Parse(rc);           //BF-35: PatientID that isn't a number
            }
            catch (Exception)
            {
                rc = "-1";
            }
            return rc;
        }

        public static DateTime ConvertStudyDate(string studyDate)
        {
            DateTime rc = DateTime.MinValue;
            try
            {
                if ((studyDate != null) && (studyDate.Length >= 8))
                    rc = new DateTime(int.Parse(studyDate.Substring(0, 4)), int.Parse(studyDate.Substring(4, 2)), int.Parse(studyDate.Substring(6, 2)), 0, 0, 0);
            }
            catch (Exception)
            {
                rc = DateTime.MinValue;
            }
            return rc;
        }

        public static string ConvertStudyDate(DateTime studyDate)
        {
            return studyDate.ToString(DicomFile.DateFormat);
        }

        private static byte[] ReadBytes(int cnt, BinaryReader binreader, BinaryWriter binwriter)
        {
            byte[] value;

            value = new byte[cnt];
            value = binreader.ReadBytes(cnt);
            binwriter.Write(value);

            return value;
        }

        private static char[] ReadChars(int cnt, BinaryReader binreader, BinaryWriter binwriter)
        {
            char[] value = binreader.ReadChars(cnt);
            binwriter.Write(value);
            return value;
        }

        private static ushort ReadUint16(ushort newSize, BinaryReader binreader, BinaryWriter binwriter)
        {
            ushort value = binreader.ReadUInt16();
            if (newSize > 0)
                binwriter.Write(newSize);
            else
                binwriter.Write(value);
            return value;
        }

        private static uint ReadUint32(uint newSize, BinaryReader binreader, BinaryWriter binwriter)
        {
            uint value = binreader.ReadUInt32(); ;
            if (newSize > 0)
                binwriter.Write(newSize);
            else
                binwriter.Write(value);
            return value;
        }

        public static int MatchSurname(string dbName, string[] fileNames)
        {
            int surnameMatch = -1;

            for (int x = 0; x < fileNames.Length; x++)
            {
                if (String.Compare(dbName, fileNames[x], true) == 0)
                {
                    surnameMatch = x;
                    break;
                }
            }
            return surnameMatch;
        }

        public static bool IsNameMatch(string patientName, string dbVolunteerName)
        {
            bool rc = false;

            if (patientName != null)
            {
                string[] fileNames = DicomFile.TokenizeString(patientName);
                string[] dbNames = DicomFile.TokenizeString(dbVolunteerName);

                int surnameMatch = -1;
                if (dbNames.Length > 0)
                    surnameMatch = DicomFile.MatchSurname(dbNames[0], fileNames);
                if (surnameMatch != -1)
                {
                    for (int x = 1; x < dbNames.Length; x++)
                    {
                        string dbFirstName = dbNames[x].ToUpper();
                        if (dbFirstName.Length > 0)
                        {
                            char dbInitial = dbFirstName[0];
                            for (int y = 0; y < fileNames.Length; y++)
                            {
                                string fileFirstName = fileNames[y].ToUpper();
                                if ((y != surnameMatch) && (fileFirstName.Length > 0))
                                {
                                    if (fileFirstName[0] == dbInitial)
                                    {
                                        rc = true;
                                        break;
                                    }
                                }
                            }
                        }
                        if (rc == true)
                            break;
                    }
                }
            }
            return rc;
        }

        private static string[] TokenizeString(string token)
        {
            string name = token.Replace(',', '^');
            name = name.Replace('.', '^');
            name = name.Replace(' ', '^');
            return name.Split(new char[1] { '^' });
        }

        //Note this should only be used to support the generation of UAT files - see DlgToolsDbAdmin.cs
        public void TestUpdateCacheFromRecord(string filename)
        {
            DicomFile.UATFileGeneration = true;
            ProcessCache(ProcessCacheAction.UpdateCacheFromRecordHeaderFields);
            WriteToDisk(new OsDatabases(), false, filename );
            DicomFile.UATFileGeneration = false;
        }
    }
}
