using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.Windows.Forms;
using System.IO;
using System.Net;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DicomMask
    {
        private byte[] _maskBuffer;         //orginal contents of image mask - monochrome
        private string _dbMaskFileLabel;           //path and file to which the masked data is saved
        private int _left;                      //actual first left column  
        private int _width;                     //actual columns in the mask
        private int _startRow;                  //actual first row
        private int _endRow;                    //actual last row
        private int _imageRows;                 //rows in image to which mask was applied
        private int _imageCols;                 //cols in image to which mask was applied
        private int _bytesPerPixel;             //1 = monochrome, 3 = colour
                                            //Config file values
        private static bool _applyMask = false;          //true if a mask is to be applied - a rectangle in image
        private static int _stdLeft = 39;                //value of mask first left column  
        private static int _stdWidth = 201;              //number of columns in the mask
        private static int _stdStartrow = 15;            //value of mask first row
        private static int _stdEndrow = 37;              //value of mask last row
        private static string _maskDir = "MaskFragments";    //directory for holding mask data removed from image - filename is DicomRef (if length == 0, then don't save mask)
        private static byte _maskImageValue = 0xF0;      //mask is almost white
        private static byte _maskPatientNameChar = 0x79; //ASCII 'Y'
        private static ushort _maskPatientNameCnt = 4;   //PatientName=YYYY
                                                //masked data file constants
        private static string _computerName = "";                       //name of the computer on which the mask was saved, if maskdirectory is not relative then it is not obtained
        public static readonly string StdExt = ".ura";                  //file type
        public static readonly string MaskFileID = "URA-Mask";          //marker for start of file
        public static readonly string MaskFileTerm = "End-URA-Mask";    //marker for end of file
        public static readonly int MaxMaskFileLength = 20;              //maximum length of file name, this allows an eight digit revision number
        public static readonly int MaxComputerNameLength = 10;           //maximum length of computer name, only important for relative mask directory
        public static readonly int MaxMaskDirLength = ImageRecord.MaxMaskPathFileLength - DicomMask.MaxMaskFileLength - DicomMask.MaxComputerNameLength;

        public DicomMask()
        {
            Reset();
        }

        public void Reset()
        {
            _maskBuffer = null;
            _dbMaskFileLabel = "";
            _left = 0;
            _width = 0;
            _startRow = 0;
            _endRow = 0;
            _imageRows = 0;
            _imageCols = 0;
            _bytesPerPixel = 0;
        }

        public static bool InitializeMaskSettings()
        {
            bool rc = false;

            try
            {
                AppSettingsSection appSettings = ExecutionTime.Configuration.AppSettings as AppSettingsSection;
                if (appSettings != null)
                {
                    KeyValueConfigurationCollection settings = appSettings.Settings;
                    string[] names = appSettings.Settings.AllKeys;
                    foreach (string name in names)
                    {
                        if (name == "maskDir")
                            _maskDir = settings[name].Value;
                        if (name == "maskImageValue")
                            _maskImageValue = byte.Parse(settings[name].Value);
                        if (name == "maskPatientNameCnt")
                            _maskPatientNameCnt = ushort.Parse(settings[name].Value);
                        if (name == "maskPatientNameChar")
                            _maskPatientNameChar = byte.Parse(settings[name].Value);
                        if (name == "maskLeftCol")
                            _stdLeft = int.Parse(settings[name].Value);
                        if (name == "maskWidth")
                            _stdWidth = int.Parse(settings[name].Value);
                        if (name == "maskStartRow")
                            _stdStartrow = int.Parse(settings[name].Value);
                        if (name == "maskEndRow")
                            _stdEndrow = int.Parse(settings[name].Value);
                        if (name == "applyMask")
                            _applyMask = (settings[name].Value == "true") ? true : false;
                    }
                }
                if (_stdStartrow > _stdEndrow)
                    throw new InvalidOperationException("invalid mask setting");
                if (_maskDir.Length > DicomMask.MaxMaskDirLength)
                    throw new InvalidOperationException("invalid MaskDir setting - too long");

                if (DicomMask.IsMaskDirRelative())
                {
                    if (Directory.Exists(ExecutionTime.Instance.InstallDir + DicomMask.MaskDir) == false)
                        Directory.CreateDirectory(ExecutionTime.Instance.InstallDir + DicomMask.MaskDir);
                    ComputerName = Dns.GetHostName();
                }
                rc = true;
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-052", UniException.Reset.TermApp, UniException.Category.Resource, ex, "config file is bad");
            }
            return rc;
        }

        public bool WriteMaskToDisk(string ShortDicomRef, Program progErrorHandler)
        {
            bool rc = false;

            if (progErrorHandler == null)
                return false;

            if (String.IsNullOrEmpty(DicomMask._maskDir))
                rc = true;
            else
            {
                string pathFilename = GetMaskPathFilename(ShortDicomRef);
                _dbMaskFileLabel = _computerName + DicomMask.MaskDir + "\\" + ShortDicomRef + DicomMask.StdExt; //v1.0.0.18 added '\\'

                StreamWriter writer = null;
                BinaryWriter binwriter = null;

                try
                {
                    if (_dbMaskFileLabel.Length > DicomMask.MaxMaskDirLength + DicomMask.MaxMaskFileLength)
                        throw new InvalidOperationException(string.Format("Path and file name too long - max 255, actual={0} [{1}]", _dbMaskFileLabel.Length, _dbMaskFileLabel));

                    if (File.Exists(pathFilename))
                        throw new InvalidOperationException(string.Format("Masked Data file already exists={0}", pathFilename));

                    if (_bytesPerPixel == 0) 
                        throw new InvalidOperationException(string.Format("DicomMask not initialized - bytes per pixel is zero"));

                    if (_maskBuffer.Length != (_bytesPerPixel * (_endRow - _startRow) * (_width)))
                        throw new InvalidOperationException(string.Format("invalid byteCnt {0} != bytes per pixel {1} * rows {2} * cols {3}", _maskBuffer.Length, _bytesPerPixel, _endRow - _startRow, _width));
//PROBLEM WPQS
                    writer = new StreamWriter(pathFilename);
                    binwriter = new BinaryWriter(writer.BaseStream);

                    char[] id = DicomMask.MaskFileID.ToCharArray();
                    binwriter.Write(id);

                    binwriter.Write(ShortDicomRef);
                    binwriter.Write(_imageRows);
                    binwriter.Write(_imageCols);
                    binwriter.Write(_startRow);
                    binwriter.Write(_endRow);
                    binwriter.Write(_left);
                    binwriter.Write(_width);
                    binwriter.Write(_bytesPerPixel);
                    binwriter.Write(_maskBuffer.Length);
                    binwriter.Write(_maskBuffer);

                    binwriter.Write(DicomMask.MaskFileTerm);

                    binwriter.Flush();
                    rc = true;
                }
                catch (Exception ex)
                {
                    progErrorHandler.Log("Error: DicomMask.WriteMaskToDisk() - {0}, Exception: {1}", pathFilename, ex.Message);
                }
                finally
                {
                    if (writer != null)
                        writer.Close();
                    if (binwriter != null)
                        binwriter.Close();
                }
            }
            return rc;
        }

        public void ReadMaskFromDisk(string ShortDicomRef)
        {
            if (String.IsNullOrEmpty(DicomMask._maskDir))
                throw new UniException("OSA-055", UniException.Reset.AbortOp, UniException.Category.System, "_maskDir is empty!");

            Reset();
   
            string pathFilename = GetMaskPathFilename(ShortDicomRef);

            try
            {
                using (StreamReader reader = new StreamReader(pathFilename))
                {
                    using (BinaryReader binreader = new BinaryReader(reader.BaseStream))
                    {
                        char[] arr = binreader.ReadChars(DicomMask.MaskFileID.Length);
                        string id = new string(arr);
                        if (id != DicomMask.MaskFileID)
                            throw new InvalidOperationException(string.Format("MaskFileID not found {0}", id));

                        string fileRef = binreader.ReadString();
                        if (fileRef != ShortDicomRef)
                            throw new InvalidOperationException(string.Format("fileRef {0} does not match {1}", fileRef, ShortDicomRef));

                        _imageRows = binreader.ReadInt32();
                        _imageCols = binreader.ReadInt32();
                        _startRow = binreader.ReadInt32();
                        _endRow = binreader.ReadInt32();
                        _left = binreader.ReadInt32();
                        _width = binreader.ReadInt32();
                        _bytesPerPixel = binreader.ReadInt32();
                        int byteCnt = binreader.ReadInt32();

                        if (byteCnt != (_bytesPerPixel * (_endRow - _startRow) * (_width)))
                            throw new InvalidOperationException(string.Format("invalid byteCnt {0} != samples per pixel {1} * rows {2} * cols {3}", byteCnt, _bytesPerPixel, _endRow - _startRow, _width));
                        if ((_startRow > _imageRows) || (_endRow > _imageRows) || (_startRow > _endRow))
                            throw new InvalidOperationException(string.Format("invalid row data; start {0} end {1} imageRows {2}", _startRow, _endRow, _imageRows));
                        if ((_left > _imageCols) || (_left + _width > _imageCols))
                            throw new InvalidOperationException(string.Format("invalid col data; left {0} width {1} imageCols {2}", _left, _width, _imageCols));

                        _maskBuffer = new byte[byteCnt];
                        _maskBuffer = binreader.ReadBytes(byteCnt);

                        if (binreader.ReadString() != DicomMask.MaskFileTerm)
                            throw new InvalidOperationException(string.Format("MaskFileTerm not found {0}", DicomMask.MaskFileTerm));
                    }
                }
            }
            catch (Exception ex)
            {
                Reset();
                throw new UniException("OSA-054", UniException.Reset.AbortOp, UniException.Category.System, ex, "cannot read {0}, {1}", pathFilename, ex.Message);
            }
            return;
        }


        public static bool IsMaskDirRelative()   //is MaskDir a network directory like //DELLDEV/ or an absolute directory like c:\bills
        {
            return ((DicomMask.MaskDir.Substring(0, 2) == "\\\\") || (DicomMask.MaskDir.Substring(1, 2) == ":\\")) ? false : true;
        }


        public void RestoreMask(byte[] image, string ShortDicomRef)
        {
            if (String.IsNullOrEmpty(DicomMask._maskDir) || (image == null) || (ShortDicomRef == null))
                throw new UniException("OSA-056", UniException.Reset.AbortOp, UniException.Category.System, "_maskDir is empty or paramas are null");
            
            ReadMaskFromDisk(ShortDicomRef);

            if ((_imageRows * _imageCols * _bytesPerPixel) > image.Length)
                throw new UniException("OSA-057", UniException.Reset.AbortOp, UniException.Category.System, "image array {0} is too small for rows {1} cols {2} bytes per pixel {3}", image.Length, _imageRows, _imageCols, _bytesPerPixel);

            int maskCnt = 0;
            for (int row = _startRow; row < _endRow; row++)
            {
                for (int col = (_left * _bytesPerPixel); col < (_left + _width) * _bytesPerPixel; col++)
                {
                    image[((row * _imageCols * _bytesPerPixel) + col)] = _maskBuffer[maskCnt++];
                }
            }
            return;
        }

        public bool ApplyMask(byte[] image, int rowsImage, int colsImage)
        {
            bool rc = false;

            if (_applyMask == true)
            {
                if ( (image == null) || ((rowsImage * colsImage) == 0) || (image.Length == 0))
                    throw new InvalidOperationException("DicomMask.ApplyMask() - invalid parameters, rows * cols = 0, image is null, or image.length = 0");

                _bytesPerPixel = image.Length / (rowsImage * colsImage);
                _startRow = (DicomMask.StartRow < rowsImage) ? DicomMask.StartRow : rowsImage;
                _endRow = (DicomMask.EndRow < rowsImage) ? DicomMask.EndRow : rowsImage;
                _left = (DicomMask.Left < colsImage) ? DicomMask.Left : colsImage;
                _width = (_left + DicomMask.Width < colsImage) ? DicomMask.Width : colsImage - _left;
                _imageRows = rowsImage;
                _imageCols = colsImage;

                _maskBuffer = new byte[((_endRow - _startRow) * _width) * _bytesPerPixel];

                int maskCnt = 0;
                for (int row = _startRow; row < _endRow; row++)
                {
                    for (int col = (_left * _bytesPerPixel); col < (_left + _width) * _bytesPerPixel; col++)
                    {
                        _maskBuffer[maskCnt++] = image[((row * colsImage * _bytesPerPixel) + col)];
                        image[((row * colsImage * _bytesPerPixel) + col)] = DicomMask.MaskImageValue;
                    }
                }
                rc = true;
            }
            return rc;
        }

        public string MaskFileLabel { get { return _dbMaskFileLabel; } }
        public static string MaskPath
        {
            get
            {
                string rc = "";
                if (DicomMask.IsMaskDirRelative())
                    rc = ExecutionTime.Instance.InstallDir + DicomMask.MaskDir;
                else
                    rc = DicomMask.MaskDir;
                return rc;
            }
        }

        public static string ComputerName
        {
            set
            {
                if (DicomMask.IsMaskDirRelative() == false)
                    _computerName = "";
                else
                {
                    if (value == null )
                        _computerName = "[null]";
                    else
                    {
                        if (value.Length > DicomMask.MaxComputerNameLength - 2)
                            _computerName = "[" + value.Substring(value.Length - DicomMask.MaxComputerNameLength - 2) + "]";
                        else
                            _computerName = "[" + value + "]";
                    }
                }
            }
            get { return _computerName; }
        }
        public static string MaskDir 
        { 
            get { return _maskDir; } 
            set { _maskDir = value; } 
        }

        public static bool MaskData 
        { 
            get { return _applyMask; } 
            set { _applyMask = value; } 

        }
        public static int Left { get { return _stdLeft; } }
        public static int Width { get { return _stdWidth; } }
        public static int StartRow { get { return _stdStartrow; } }
        public static int EndRow { get { return _stdEndrow; } }

        public static byte MaskImageValue { get { return _maskImageValue; } }
        public static ushort MaskPatientNameCnt { get { return _maskPatientNameCnt; } }
        public static byte MaskPatientNameChar { get { return _maskPatientNameChar; } }

        public static string GetMaskPathFilename(string ShortDicomRef)
        {
            return DicomMask.MaskPath + "\\" + ShortDicomRef + DicomMask.StdExt; //v1.0.0.18 added '\\' 
        }

    }
}
