using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DlgProcDICOMCorrection
    {
        private DicomFile _file;
        private DicomFile[] _files;
        private OsDatabases _databases;
        private Program _progErrorHandler;
        private string  _doOKMessage;
        private MessageBoxButtons _doOKMessageBoxButtons;

        private string  _tmpPatientName;
        private string  _tmpVolunteerRef;
        private DateTime _tmpAppointmentDate;

        public DlgProcDICOMCorrection(object correction, OsDatabases databases, Program progErrorHandler)
        {
            Examination exam = correction as Examination;
            if (exam == null)
            {
                SelectDicomFile(correction as DicomFile);
                _files = new DicomFile[1];
                _files[0] = _file;
            }
            else
            {
                _files = exam.GetList();
                if (_files.Length > 0)
                    SelectDicomFile(_files[0]);
            }

            if ((progErrorHandler == null) || (databases == null) || (_file == null) || (_file.Record == null))
                throw new UniException("OSA-013", UniException.Reset.AbortOp, UniException.Category.Program, "Invalid parameters: file, file.record, progErrorHandler or Databases are null");

            _databases = databases;
            _progErrorHandler = progErrorHandler;

            _doOKMessage = "";
            _doOKMessageBoxButtons = MessageBoxButtons.OK;
        }

        public void SelectDicomFile(DicomFile file)
        {
            if ( file == null )
                throw new UniException("OSA-071", UniException.Reset.AbortOp, UniException.Category.Program, "Invalid parameter: file is null");
            _file = file;
        }

        public DicomFile[] GetDicomFiles()
        {
            return _files;
        }

        public DialogResult DoOK(string patientNameText, string VolRefText, DateTime Appointment)
        {
            DialogResult rc = DialogResult.None;

            _doOKMessage = "";
            _doOKMessageBoxButtons = MessageBoxButtons.OK;

            _file.Record.PatientName = patientNameText;
            _file.Record.VolunteerRef = VolRefText;
            _file.Record.AppointmentDate = Appointment;
            _file.Record.ResetUnspecifiedError();    //remove unspecified error so it might now be imported if other header items are ok
            _file.Record.SetValidationFlags(_databases, _progErrorHandler);

            if ((_file.Record.IsAnyValidationFlagSet() == false) && (_file.Record.IsValidDicomRef()))
            {
                rc = DialogResult.OK;
            }
            else
            {
                if (_file.Record.NameMismatch)
                    _doOKMessage = Resources.StrMsgBoxNameMismatch;
                else
                {
                    if (_file.Record.IsValidDicomRef() == false)
                        _doOKMessage = Resources.StrMsgBoxInvalidDicomRef;
                    else
                    {
                        _doOKMessageBoxButtons = MessageBoxButtons.YesNo;

                        if (_file.Record.Orphan)
                            _doOKMessage = Resources.StrMsgBoxOrphan + Utils.StrLFCR + Utils.StrLFCR;
                        if (_file.Record.Duplicate)
                            _doOKMessage += Resources.StrMsgBoxDuplicate + Utils.StrLFCR + Utils.StrLFCR;
                        if (_file.Record.UnspecifiedError)  //this should never happen as ResetUnspecifiedError() is called above
                            _doOKMessage += Resources.StrMsgBoxUnspecError + Utils.StrLFCR + Utils.StrLFCR;
                    }
                }
            }
            return rc;
        }

        public bool SetDetailsForAllFilesInExam(DicomFile correctFile)
        {
            bool rc = false;

            if (correctFile != null)
            {
                foreach (DicomFile file in _files)
                {
                    if (file != _file)
                    {
                        file.Record.PatientName = correctFile.Record.PatientName;
                        file.Record.VolunteerRef = correctFile.Record.VolunteerRef;
                        file.Record.AppointmentDate = correctFile.Record.AppointmentDate;
                        file.Record.ResetUnspecifiedError();    //remove unspecified error so it might now be imported if other header items are ok
                        file.Record.SetValidationFlags(_databases, _progErrorHandler);
                    }
                }
                rc = true;
            }
            return rc;
        }

        public void SaveRecordValues()
        {
            _tmpPatientName = _file.Record.PatientName;
            _tmpVolunteerRef = _file.Record.VolunteerRef;
            _tmpAppointmentDate = _file.Record.AppointmentDate;
        }

        public void RestoreRecordValues()
        {
            _file.Record.PatientName = _tmpPatientName;
            _file.Record.VolunteerRef = _tmpVolunteerRef;
            _file.Record.AppointmentDate = _tmpAppointmentDate;
            _file.Record.SetValidationFlags(_databases, _progErrorHandler); //restore error flags
        }

        public string PatientNameLabel
        {
            get { return _file.Record.PatientName; }
        }
        public string VolunteerRefLabel
        {
            get { return _file.Record.VolunteerRef; }
        }
        public DateTime AppointmentDateValue
        {
            get { return _file.Record.AppointmentDate; }
        }

        public DicomFile SelectedDicomFile
        {
            get { return _file; }
        }

        public string ImagePathFilename
        {
            get { return _file.Record.DicomPathFilename; }
        }

        public MessageBoxButtons DoOKMessageButtons
        {
            get { return _doOKMessageBoxButtons; }
        }

        public string DoOKMessage
        {
            get { return _doOKMessage; }
        }

        public bool Duplicate
        {
            get { return _file.Record.Duplicate; }
        }

        public static string GetSurname(string dicomFullName, bool surnameFirst)
        {
            string rc = "";
            if (dicomFullName != null)
            {
                string[] names = dicomFullName.Split(new char[1] { '^' });
                if (names.Length > 0)
                {
                    if (surnameFirst)
                        rc = names[0];
                    else
                        rc = names[names.Length - 1];
                }
            }
            return rc;
        }
        public static string GetFirstNames(string dicomFullName, bool surnameFirst)
        {
            string rc = "";
            if (dicomFullName != null)
            {
                string[] names = dicomFullName.Split(new char[1] { '^' });
                int firstFirstName = 0;
                int lastFirstName = names.Length - 1;
                if (surnameFirst)
                {
                    firstFirstName = 1;
                    lastFirstName = names.Length;
                }
                StringBuilder result = new StringBuilder(rc);
                for (int x = firstFirstName; x < lastFirstName; x++)
                    result.Append(names[x] + ((x != lastFirstName - 1) ? " " : ""));
                rc = result.ToString();
            }
            return (rc.Length == 0) ? "%" : rc;
        }

        public string DicomRefLabel
        {
            get
            {
                return _file.Record.DicomRef + ((_file.Record.IsValidDicomRef() == false) ? Resources.StrInvalidWarning : "");
            }
        }

        public string ErrorText
        {
            get
            {
                return _file.Record.ValidationFlagDetails + _file.Record.UnspecifiedErrorDetails;
            }
        }

        public OsDatabases Databases
        {
            get
            {
                return _databases;
            }
        }

    }
}
