using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public abstract class DlgProcFileImport
    {
        internal OsDatabases db;
        internal ListBox imageLB;
        internal ListBox correctionsLB;
        internal ProgressBar workPB;
        internal Label progressLabel;
        internal Program errorHandle;
        internal Control dlgCtrl;
        internal int delayPeriod;
        internal SelectionMode selectionModeImageList;
        internal SelectionMode selectionModeCorrectionList;
        internal DlgBrowser dlgBrowser;

        public enum ImportMode
        {
            None = 0,
            IndividualFile = 1,
            AllFilesInTree = 2,
            ExaminationCollections = 3,
        }

        public abstract string DialogTitle { get; }
        public abstract int ReadFiles(string Location, string regex);


        public static Int32 ImportModeToInt32(ImportMode value)
        {
            if (value == ImportMode.IndividualFile)
                return 1;
            if (value == ImportMode.ExaminationCollections)
                return 3;
            return 2;
        }

        public static ImportMode Int32ToImportMode(Int32 value)
        {
            if (value == 1)
                return ImportMode.IndividualFile;
            if (value == 3)
                return ImportMode.ExaminationCollections;
            return ImportMode.AllFilesInTree;
        }

        public static DlgProcFileImport CreateDlgProc(ImportMode mode, OsDatabases databases, ListBox imageListBox, ListBox correctionsListBox, ProgressBar workProgressBar, Label progressTextLabel, int delay, Program errorHandler)
        {
            DlgProcFileImport rc = null;
            switch (mode)
            {
                case DlgProcFileImport.ImportMode.AllFilesInTree:
                    rc = new DlgProcFileImportTree(databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler);
                    break;
                case DlgProcFileImport.ImportMode.ExaminationCollections:
                    rc = new DlgProcFileImportExam(databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler);
                    break;
                default:
                    rc = new DlgProcFileImportFile(databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler);
                    break;
            }
            return rc;
        }

        public DlgBrowser BrowserDlg
        { 
            set { dlgBrowser = value; }
            get { return dlgBrowser as DlgBrowser; } 
        }


        protected DlgProcFileImport(OsDatabases databases, ListBox imageListBox, ListBox correctionsListBox, ProgressBar workProgressBar, Label progressTextLabel, int delay, Program errorHandler)
        {
            db = databases;
            imageLB = imageListBox;
            correctionsLB = correctionsListBox;
            workPB = workProgressBar;
            if (workPB != null)
            {
                workPB.Step = 1;
                workPB.Minimum = 0;
            }
            progressLabel = progressTextLabel;
            errorHandle = errorHandler;
            delayPeriod = delay;
            dlgCtrl = (imageListBox != null) ? imageListBox.Parent : null;
            selectionModeImageList = (imageListBox != null) ? imageListBox.SelectionMode : SelectionMode.One;
            selectionModeCorrectionList = (correctionsListBox != null) ? correctionsListBox.SelectionMode : SelectionMode.One;
        }

        public virtual bool acceptClick(ListBox correctionsListbox, ListBox imageListbox)
        {
            bool rc = false;

            if ((correctionsListbox != null) && (imageListbox != null))
            {
                int selected = correctionsListbox.SelectedIndex;
                if (selected != -1)
                {
                    DicomFile correctedFile = correctionsListbox.Items[selected] as DicomFile;
                    if (correctedFile != null)
                    {
                        int index = -1;
                        index = imageListbox.Items.Add(correctedFile);
                        imageListbox.SelectedIndex = index;

                        int lastRemovedItem = DlgUtils.RemoveSelectedItemFromListBox(correctionsListbox, -1);
                        if ((correctionsListbox.Items.Count > 0) && (lastRemovedItem != -1))
                            correctionsListbox.SelectedIndex = (lastRemovedItem == correctionsListbox.Items.Count) ? lastRemovedItem - 1 : lastRemovedItem;
                        rc = true;
                    }
                }
            }
            return rc;
        }

        public virtual String OpenBrowseDlg(String LastImportDir, Control fileLocation)
        {
            String rc = "";

            if (dlgBrowser != null)
            {
                dlgBrowser.SelectedDir = LastImportDir;
                dlgBrowser.ShowNewFolderButton = false;
                if ( dlgBrowser.ShowDialog() == DialogResult.OK)
                {
                    rc = dlgBrowser.SelectedDir;
                    if (fileLocation != null)
                        fileLocation.Text = dlgBrowser.SelectedDir;
                }
            }
            return rc;
        }

        public virtual bool GetAcceptButtonStateForSelection(object selectedItem)
        {
            bool rc = false;
            DicomFile file = selectedItem as DicomFile;
            if (file != null)
            {
                if ((file.Record.NameMismatch != true) && (file.Record.UnspecifiedError != true))
                    rc = true;
            }
            return rc;
        }

        public virtual String[] GetFilenamesForSelectedItem(object selectedItem)
        {
            String[] rc = null;
            DicomFile file = selectedItem as DicomFile;
            if (file != null)
            {
                rc = new String[1];
                rc[0] = file.Record.DicomPathFilename;
            }
            return rc;
        }

        public virtual int ImportFiles(string diskRef, DateTime captureDate)
        {
            int rc = 0;

            DisplayProgressStart(Resources.StrImportingImages, 0, imageLB.Items.Count);
           
            string errorText = Utils.StrLFCR;
            int errorCnt = 0;

            DicomFile[] files = new DicomFile[imageLB.Items.Count];
            imageLB.Items.CopyTo(files, 0 );
            for (int index = 0; index < files.Length; index++)
            {
                DisplayProgressStep(Resources.StrImportingFile);
                
                DicomFile file = files[index];
                file.Record.DiskLabel = diskRef;
                file.Record.ArchiveDate = captureDate;
                if (db.StoreImageRecord(file, errorHandle) == true)
                {
                    imageLB.Items.Remove(file);
                    rc++;
                }
                else
                {
                    errorText += file.ToString() + " : " + file.Record.UnspecifiedErrorDetails + Utils.StrLFCR;
                    errorCnt++;
                }
            }
            DisplayProgressEnd(rc, (rc == 1) ? Resources.StrFileImported : Resources.StrFilesImported);

            if (errorCnt > 0)
                throw new UniException("OSA-036", UniException.Reset.AbortOp, UniException.Category.System, "failed to import {0} files: {1}", errorCnt, errorText);

            return rc;
        }

        public int ImageListCnt
        {
            get { return (imageLB == null) ? -1 : imageLB.Items.Count; }
        }

        public int CorrectionsListCnt
        {
            get { return (correctionsLB == null) ? -1 : correctionsLB.Items.Count; }
        }

        public bool DiskRefIsValid(string DiskRef)
        {
            bool rc = false;
            if ((DiskRef != null) && (DiskRef.Length > 0) && (DiskRef != Resources.StrReservedDiskLabel))
            {
                ImageRecord[] list = db.GetAllImageDetailsForDiskRef(DiskRef);
                if (list.Length == 0)
                    rc = true;
            }
            return rc;
        }

        protected void DisplayProgressStart(string label, int startValue, int maxValue)
        {
            progressLabel.Text = label;
            if (workPB != null)
            {
                workPB.Maximum = maxValue;
                workPB.Value = startValue;
            }
            selectionModeImageList = imageLB.SelectionMode;
            imageLB.SelectionMode = SelectionMode.None;
            selectionModeCorrectionList = correctionsLB.SelectionMode;
            correctionsLB.SelectionMode = SelectionMode.None;

            if (dlgCtrl != null)
                dlgCtrl.Refresh();
            if (delayPeriod == -1)
            {
                imageLB.BeginUpdate();
                correctionsLB.BeginUpdate();
            }
        }

        protected void DisplayProgressEnd(int itemsProcessed, string itemDescription)
        {
            progressLabel.Text = itemsProcessed.ToString() + " " + itemDescription;
            if (workPB != null)
                workPB.Value = 0;
            imageLB.SelectionMode = selectionModeImageList;
            correctionsLB.SelectionMode = selectionModeCorrectionList;
            if (delayPeriod == -1)
            {
                imageLB.EndUpdate();
                correctionsLB.EndUpdate();
            }
            if (dlgCtrl != null)
                dlgCtrl.Refresh();
        }

        protected void DisplayProgressStep(string activity)
        {
            if (workPB != null)
            {
                workPB.PerformStep();
                if ((dlgCtrl != null) && (delayPeriod != -1))
                {
                    progressLabel.Text = activity + workPB.Value.ToString() + Resources.StrOf + workPB.Maximum.ToString();
                    dlgCtrl.Refresh();
                }
            }
            if (delayPeriod > 0)
                System.Threading.Thread.Sleep(delayPeriod);
        }
    }
}
