using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class ImageRecord
    {
                                            //fields that identify the record
        private string _imageID;                //unique id, if record created from database, guid
        private string _dicomPathFilename;      //path and filename of dicom file, if record created from file - not stored in database
                                            //fields originating from Dicom Header (or Corrected by user) - Header Fields
        private string _dicomRef;               //Header Field: StudyInstanceUID (0008,0018), varchar(64)    
        private DateTime _appointmentDate;      //Header Field: StudyDate (0008,0020), datetime
        private string _volunteerRef;           //Header Field: PatientID (0010,0020), int
        private string _patientName;            //Header Field: PatientName (0010,0010), surname=varchar(30), firsnames=varchar(50) 
                                            //fields corresponding to the Dicom File 
        private byte[] _dicomFile;              //the entire Dicom file, header + image(s), image
        private int _dicomFileSize;             //size of _dicomFile - not stored in database
                                            //fields orginating from when the dicom file was imported
        private string _maskFileLabel;          //location of image mask fragment - shared network directory, varchar(255)
        private DateTime _archiveDate;          //date the dicom file was imported, datetime
        private string _archiveOperator;        //person logged-on when dicom file was imported, varchar(50)
        private string _diskLabel;              //label of the disk from which the dicom file was read, varchar(50)
        private string _unspecifiedErrorText;   //errors detected when creating record from file, not stored in database
        private FlagMask _flags;                //HeaderFlags: Duplicate, NameMismatch, Orphan, Unspecified error, varbin(10)
        private string _duplicateDetails;       //ImageID for record already in database with matching _dicomRef (like revision #) - BF-33

        public static readonly int MinDicomRefLength = 34;                     //display all following character as short dicom ref
        public static readonly int MaxDicomRefLength = 64;                     //defined by Dicom standard
        public static readonly int MaxSurnameLength = 30;                      //defined by existing UKCTOCS database V_Volunteers.V_Surname
        public static readonly int MaxFirstNamesLength = 50;                   //defined by existing UKCTOCS database V_Volunteers.V_FirstNames
        public static readonly int MaxMaskPathFileLength = 255;                //set by image table, MaskFile
        public static readonly int MaxUserNameLength = 50;                     //set by image table, StoredBy
        public static readonly int MaxDiskLabelLength = 50;                    //set by image table, DiskRef
        public static readonly int FlagsLength = 10;                           //set by image table, Flags


        public static readonly string DateFormat = "yyyy-MM-dd";               //format of dates when displayed as strings
        public static readonly string CranfieldCountryOrgCode = "826.999999";  //not officially granted, if this changes then update this part of the DicomRef in all Image database records 
        public static readonly int    CranfieldRevisionPosition = 10;          //position of revision number in Cranfield's DicomRef (Cranfield OSI.ANSIBODY.COUNTRY.ORGCODE.DEVICETYPE.SERIALNO.1.STUDYDATE,STUDYHR,6or7DIGITUNIQUECODE.SERIESNO.IMAGENO.Rev)

        private enum FlagMask
        {
            Clear = 0,
            Orphan = 1,                     //validation flag - no matching scandate record in ukctocs database for this record's volunteer ref and appointment date
            Duplicate = 2,                  //validation flag - identical image.ImageID found in URA database, so this record will be updated
            NameMismatch = 4,               //validation flag - no matching volunteer record in ukctocs database for this record's VolunteerRef and PatientName
            UnspecifiedError = 8,           //some error was detected when this file was read - prior to importing this file the error details are in _unspecifiedErrorText
        }

        public ImageRecord()
        {
            Init("");
        }

        public ImageRecord(string pathFilename)
        {
            Init(pathFilename);
        }

        public override string ToString()
        {
            string name = "";
            if (_dicomPathFilename.Length == 0)
                name = CreateFilename();
            else
            {
                name = FlagDetails;
                name += _dicomPathFilename;
            }
            return name;
        }

        public string FlagDetails
        {
            get
            {
                string name = "[";

                if (UnspecifiedError == true)
                    name += "XXX";
                else
                {
                    name += (Orphan) ? "O" : "-";
                    name += (Duplicate) ? "D" : "-";
                    name += (NameMismatch) ? "N" : "-";
                }
                name += "] ";
                return name;
            }
        }

        public string CreateFilename()
        {
            string shortDicomRef = (GetShortDicomRef().Length == 0) ? "" : "-" + GetShortDicomRef();
            string name = VolunteerRef + "-" + AppointmentDate.ToString(ImageRecord.DateFormat).Replace('/', '-') + shortDicomRef;
            return name;
        }

        public string Details
        {
            get
            {
                string details = "";

                details += _imageID;
                details += Utils.StrLFCR + "Filesize=[" + _dicomFileSize.ToString() + "] Pathname=" + ImageRecord.DashOrValue(_dicomPathFilename);
                details += Utils.StrLFCR + "DicomRef=" + ImageRecord.DashOrValue(_dicomRef);
                details += Utils.StrLFCR + "AppointmentDate=" + _appointmentDate.ToString();
                details += Utils.StrLFCR + "VolunteerRef=" + ImageRecord.DashOrValue(_volunteerRef);
                details += Utils.StrLFCR + "PatientName=" + ImageRecord.DashOrValue(_patientName);
                details += Utils.StrLFCR + "MaskPathFilename=" + ImageRecord.DashOrValue(_maskFileLabel);
                details += Utils.StrLFCR + "ArchiveDate=" + _archiveDate.ToString();
                details += Utils.StrLFCR + "ArchiveOperator=" + ImageRecord.DashOrValue(_archiveOperator);
                details += Utils.StrLFCR + "DiskLabel=" + ImageRecord.DashOrValue(_diskLabel);
                details += Utils.StrLFCR + "Flags=" + FlagDetails + "Unspec Error Details=" + ImageRecord.DashOrValue(UnspecifiedErrorDetails);

                return details;
            }
        }

        private static string DashOrValue(string value)
        {
            return (value.Length == 0) ? "-" : value;
        }

        private void Init(string pathFilename)
        {
            _imageID = "";                          //not yet associated with an URA database image record
            _dicomPathFilename = pathFilename;      

            ClearHeaderFields();
            ClearDicomFileCache();

            _maskFileLabel = "";
            _archiveDate = DateTime.MinValue;
            _archiveOperator = "(unknown)"; ;
            _diskLabel = "";
            _flags = 0;
            ResetValidationFlags();
            ResetUnspecifiedError();
            _duplicateDetails = "";
        }

        public void Clear()
        {
            Init("");
        }

        public void ClearHeaderFields()
        {
            _dicomRef = "";
            _appointmentDate = DateTime.MinValue;
            _volunteerRef = "";
            _patientName = "";
        }

        public bool HeaderFieldsInitiated()
        {
            bool rc = false;
            if ((_appointmentDate != DateTime.MinValue) && (IsValidDicomRef() == true) && (_volunteerRef.Length > 0))
            {
                if (_patientName.Length > 0) 
                    rc = true;
            }
            return rc;
        }

        public void ResetValidationFlags()
        {
            _flags = TurnBitOff(_flags, FlagMask.Orphan);
            _flags = TurnBitOff(_flags, FlagMask.Duplicate);
            _flags = TurnBitOff(_flags, FlagMask.NameMismatch);
        }

        public bool SetValidationFlags(OsDatabases db, Program progErrorHandler)
        {
            bool rc = false;

            if (db != null)
            {
                NameMismatch = (DicomFile.IsNameMatch(PatientName, db.GetNameFromVolunteerRef(this, progErrorHandler))) ? false : true;
                Orphan = (db.DoesScanDateExistForVolunteerRef(this, progErrorHandler)) ? false : true;
                Duplicate = GetExistingRecordDetails(db, progErrorHandler);

                if (IsAnyValidationFlagSet() == false)
                    rc = true;
            }
            return rc;
        }

        public bool GetExistingRecordDetails(OsDatabases db, Program progErrorHandler)  //BF-33
        {
            bool rc = false;

            if (db != null)
            {
                ImageRecord duplicateRecord = new ImageRecord();
                duplicateRecord.DicomRef = DicomRef;

                string duplicateImageID = db.GetImageIDForDicomRef(duplicateRecord, progErrorHandler);
                if (duplicateImageID.Length > 0)
                {
                    ImageID = duplicateImageID;
                    db.LoadImageDetailsForImageID(duplicateRecord);
                    UpdateDicomRef(duplicateRecord.DicomRef);

                    _duplicateDetails = Resources.StrDupRecordFound;
                    _duplicateDetails += Resources.StrDiskLabel + duplicateRecord.DiskLabel + ", ";
                    _duplicateDetails += Resources.StrArchiveDate + duplicateRecord.ArchiveDate.ToString() + ", ";
                    _duplicateDetails += Resources.StrArchiveOperator + duplicateRecord.ArchiveOperator;

                    rc = true;
                }
            }
            return rc;
        }

        public bool IsAnyValidationFlagSet()
        {
            bool rc = true;
            if ((Orphan == false) && (Duplicate == false) && (NameMismatch == false) )
                rc = false;
            return rc;
        }

        public string ValidationFlagDetails
        {
            get
            {
                string rc = "";
                if (Orphan)
                    rc += Resources.StrErrorOrphan + Utils.StrLFCR;
                if (Duplicate)
                    rc += Resources.StrErrorDuplicate + DuplicateDetails + Utils.StrLFCR;
                if (NameMismatch)
                    rc += Resources.StrErrorNameMismatch + Utils.StrLFCR;
                return rc;
            }
        }

        public string UnspecifiedErrorDetails
        {
            get
            {
                string rc = "";
                if (UnspecifiedError == true)
                    rc += Resources.StrErrorUnspec + _unspecifiedErrorText;
                return rc;
            }
        }


        //********************************************************************************************************************
        //SOPInstanceUID (0008,0018)
        //1.2.410.200001.1.9900.1.20050531132655297.1.1
        //SonoView  OSI.ANSIBODY.COUNTRY.ORGCODE.DEVICETYPE.SERIALNO.1.STUDYDATE,STUDYHR,6or7DIGITUNIQUECODE.SERIESNO.IMAGENO 
        //          1  .2       .410    .200001  .1         .9900    .1.20050531 ,11     ,32655297           .1       .1
        //Cranfield OSI.ANSIBODY.COUNTRY.ORGCODE.DEVICETYPE.SERIALNO.1.STUDYDATE,STUDYHR,6or7DIGITUNIQUECODE.SERIESNO.IMAGENO.Rev
        //          1  .2       .826    .999999  .1         .9900    .1.20050531 ,11     ,32655297           .1       .1     .0
        //[6or7DIGITUNIQUECODE.SERIESNO.IMAGENO.Rev] is short dicom ref
        //
        //Note 1: Cranfield University should apply to BSI (UK ANSI Body) for its own organisational code. However, given
        //that all the ultrasound scanners are installed and will not be replaced for the duration of the trail
        //it seems reasonable to apply an unofficial organisational code (999999) without there being an danager of a conflict. 
        //We also append a revision number to the end of the string which is incremented each time the image (or record)
        //is updated. The advantage of this approach is that the original SOPInstanceUID can be reconstituted, if ever necessary
        //It may be helpful to consider the following scenario from the unit test - UpdateRevisionTest:
        //  1. import SonoView file A DicomRef = xxxx.0 -> (new) image record DicomRef = xxxx.1 (with image mask and potentially different studydate, patientid)
        //  2. import SonoView file A DicomRef = xxxx.0 -> (update)image record DicomRef = xxxx.2 (potentially with different studydate, patientid)
        //  3. export image record DicomRef = xxxx.2 -> Cranfield file DicomRef = xxxx.2
        //  4. import Cranfield file xxxx.2 -> (update) image record DicomRef = xxxx.3 (potentially with different studydate, patientid)
        //  5. import SonoView file A DicomRef = xxxx.0 -> (update) image record DicomRef = xxxx.4 (potentially with different studydate, patientid)
        //********************************************************************************************************************

        public static string GetCranfieldDicomRef(string value) //invoked whenever the DicomRef is set 
        {
            if (value != null)
            {
                if (value.Length < ImageRecord.MinDicomRefLength)
                    value = "";
                else
                {
                    string CountryOrgCode = value.Substring(4, 10);
                    if (CountryOrgCode != ImageRecord.CranfieldCountryOrgCode)
                        value = value.Replace(CountryOrgCode, ImageRecord.CranfieldCountryOrgCode) + ".0";
                    if (value.Length > ImageRecord.MaxDicomRefLength)
                        value = "";
                }
            }
            return value;
        }

        public static int GetRevisionNumber(string dicomRef) //BF-33
        {
            int rc = -1;
            if (dicomRef != null)
            {
                try
                {
                    int revIndex = -1;
                    for (int dotCount = 0; dotCount < ImageRecord.CranfieldRevisionPosition; dotCount++)
                        revIndex = dicomRef.IndexOf(".", revIndex + 1);
                    if (revIndex > -1)
                        rc = int.Parse(dicomRef.Substring(revIndex + 1));
                }
                catch (Exception)
                {
                }
            }
            return rc;
        }

        public string UpdateDicomRef(string existingDicomRef)
        {
            string rc = ""; 
            try
            {
                int existingRevision = ImageRecord.GetRevisionNumber(existingDicomRef);  //BF-33
                int revIndex = _dicomRef.LastIndexOf(".");
                if ((existingRevision == -1) || (revIndex == -1))
                    throw new InvalidOperationException(Resources.StrInvalidDicomRefRevision);
                existingRevision++;
                rc = _dicomRef.Remove(revIndex) + "." + existingRevision.ToString();
                if (rc.Length > ImageRecord.MaxDicomRefLength)
                    throw new InvalidOperationException(Resources.StrInvalidDicomRefTooLong);
                _dicomRef = rc;
            }
            catch (Exception ex)
            {
                UnspecifiedErrorText = Resources.StrInvalidDicomRef + _dicomRef + "," + ex.Message;
            }
            return rc;
        }

        public string DicomRefMatchStr
        {                               //replace revision number with .% as the database only holds the latest revision of any file
            get
            {
                string rc = _dicomRef;

                int trim = rc.LastIndexOf(".");
                if (trim > -1)
                    rc = rc.Remove(trim) + ".%";

                return rc;
            }
        }

        public static bool MatchDicomRefAnyRevision(string ref1, string ref2)
        {
            bool rc = false;
            if ((ref1 != null) && (ref2 != null))
            {
                int trim1 = ref1.LastIndexOf(".");
                int trim2 = ref2.LastIndexOf(".");
                if ((trim1 > -1) && (trim2 > -1))
                {
                    if (ref1.Remove(trim1) == ref2.Remove(trim2))
                        rc = true;
                }
            }
            return rc;
        }

        public bool IsValidDicomRef()
        {
            bool rc = false;
            if ((_dicomRef.Length >= ImageRecord.MinDicomRefLength) && ((_dicomRef.Length <= ImageRecord.MaxDicomRefLength)))
                rc = true;
            return rc;
        }

        public static string GetShortDicomRef(string DicomRef)
        {
            string rc = (DicomRef != null) ? DicomRef : "(null)";
            if (rc.Length > ImageRecord.MinDicomRefLength)
                rc = rc.Substring(ImageRecord.MinDicomRefLength);
            return rc;
        }

        public string GetShortDicomRef()
        {       
            return GetShortDicomRef(_dicomRef);
        }

        public bool IsDicomFileCached() { return (_dicomFile == null) ? false : true; }
        public void ClearDicomFileCache() { SetDicomFileCache(null); }
        public byte[] GetDicomFileCache()
        {
            return _dicomFile;
        }
        public int SetDicomFileCache(byte[] value)
        {
            _dicomFile = value;
            _dicomFileSize = (_dicomFile == null) ? 0 : _dicomFile.Length;
            return _dicomFileSize;
        }

        public int DicomFileSize
        {
            set { _dicomFileSize = value; }
            get { return _dicomFileSize; }
        }
        
        public bool Orphan
        {
            get { return ((_flags & FlagMask.Orphan) != 0) ? true : false; }
            set { _flags = (value == true) ? TurnBitOn(_flags, FlagMask.Orphan) : TurnBitOff(_flags, FlagMask.Orphan); }
        }

        public bool Duplicate
        {
            get { return ((_flags & FlagMask.Duplicate) != 0) ? true : false; }
            set { _flags = (value == true) ? TurnBitOn(_flags, FlagMask.Duplicate) : TurnBitOff(_flags, FlagMask.Duplicate); }
        }

        public bool NameMismatch
        {
            get { return ((_flags & FlagMask.NameMismatch) != 0) ? true : false; }
            set { _flags = (value == true) ? TurnBitOn(_flags, FlagMask.NameMismatch) : TurnBitOff(_flags, FlagMask.NameMismatch); }
        }
        public bool UnspecifiedError
        {
            get { return ((_flags & FlagMask.UnspecifiedError) != 0) ? true : false; }
        }

        public void ResetUnspecifiedError()
        {
            SetUnspecifiedError(false);
            _unspecifiedErrorText = Resources.StrErrorNotSet;
        }
        private void SetUnspecifiedError(bool value)
        {
            _flags = (value == true) ? TurnBitOn(_flags, FlagMask.UnspecifiedError) : TurnBitOff(_flags, FlagMask.UnspecifiedError); 
        }

        public string UnspecifiedErrorText
        {
            set
            {
                _unspecifiedErrorText = value;
                SetUnspecifiedError(true);
            }
            get { return _unspecifiedErrorText; }
        }

        private static FlagMask TurnBitOn(FlagMask value, FlagMask bitToTurnOn)
        {
            return (value | bitToTurnOn);
        }
        private static FlagMask TurnBitOff(FlagMask value, FlagMask bitToTurnOff)
        {
            return (value & ~bitToTurnOff);
        }

        public byte[] GetFlags()
        {
            byte[] ret = new byte[ImageRecord.FlagsLength];
            if (Orphan == true)
                ret[0] = 1;
            if (Duplicate == true)
                ret[1] = 1;
            if (NameMismatch == true)
                ret[2] = 1;
            if (UnspecifiedError == true)
                ret[3] = 1;
            return ret;
        }
        public void SetFlags(byte[] value)
        {
            Orphan = (value[0] == 1) ? true : false;
            Duplicate = (value[1] == 1) ? true : false;
            NameMismatch = (value[2] == 1) ? true : false;
            SetUnspecifiedError((value[3] == 1) ? true : false);
        }

        public string ImageID
        {
            get { return _imageID; }
            set { _imageID = value; }
        }
        public DateTime AppointmentDate
        {
            get { return _appointmentDate; }
            set { _appointmentDate = value; }
        }
        public string VolunteerRef
        {
            get { return _volunteerRef; }
            set { _volunteerRef = value; }
        }
        public string PatientName
        {
            get { return _patientName; }
            set { _patientName = value; }
        }

        public string DicomRef
        {
            get { return _dicomRef; }
            set { _dicomRef = ImageRecord.GetCranfieldDicomRef(value); }
        }
        public string DicomPathFilename
        {
            get { return _dicomPathFilename; }
            set { _dicomPathFilename = value; }
        }
        public string MaskFileLabel
        {
            get { return _maskFileLabel; }
            set { _maskFileLabel = value; }
        }

        public DateTime ArchiveDate
        {
            get { return _archiveDate; }
            set { _archiveDate = value; }
        }
        public string ArchiveOperator
        {
            get { return _archiveOperator; }
            set { _archiveOperator = value; }
        }
        public string DiskLabel
        {
            get { return _diskLabel; }
            set { _diskLabel = value; }
        }
        public string DuplicateDetails
        {
            get { return _duplicateDetails; }
        }
    }
}
