using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Configuration;


using AxezDICOMax;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class ImageRecordTreeView : TreeView
    {
        private TreeNode _rootTreeNode;
        private OsDatabases _databases;
        private AxezDICOMX _axezDICOMX;
        private string _tempFile;
        private RegionalCentres _centres;
        private int _totalImagesInTree;
        private ToolStripStatusLabel _statusLabelToolStrip;
        private ToolStripMenuItem _deleteMenu;
        private Program _progErrorHandler;
        private TreeNode _unknownCentreNode;

        private bool _initDone;

        private static readonly string UnknownCentreNumber = "99";
        private static readonly string TagVolunteerRef = "VolunteerRef";
        private static readonly string TagScanDate = "ScanDate";

        public ImageRecordTreeView()
        {
            _databases = null;
            _axezDICOMX = null;
            _tempFile = "";
            _centres = null;
            _totalImagesInTree = 0;
            _statusLabelToolStrip = null;
            _progErrorHandler = null;
            _initDone = false;
            _unknownCentreNode = null;
        }

        public void Init(AxezDICOMX axezDICOMX, OsDatabases databases, RegionalCentres centres, string tmpFilePathName, ToolStripStatusLabel statusLabelToolStrip, ToolStripMenuItem deleteMenu, Program progErrorHandler)
        {
            _axezDICOMX = axezDICOMX;
            if (_axezDICOMX != null)
            {
                _axezDICOMX.DCMtoolbar = false;
                _axezDICOMX.CtlVisible = false;
            }
            Init(databases, centres, tmpFilePathName, statusLabelToolStrip, deleteMenu, progErrorHandler);
        }

        public void Init(OsDatabases databases, RegionalCentres centres, string tmpFilePathName, ToolStripStatusLabel statusLabelToolStrip, ToolStripMenuItem deleteMenu, Program progErrorHandler)
        {
            _progErrorHandler = progErrorHandler;
            _deleteMenu = deleteMenu;
            _tempFile = tmpFilePathName;
            _databases = databases;
            _centres = centres;
            _statusLabelToolStrip = statusLabelToolStrip;

            if ((_progErrorHandler == null) || (_deleteMenu == null) || (_tempFile == null) || (_tempFile.Length == 0) || (_databases == null) || (_centres == null) || (_statusLabelToolStrip == null))
                throw new UniException("OSA-062", UniException.Reset.TermApp, UniException.Category.Program, "Init passed null value");

            _deleteMenu.Enabled = false;

            ImageList myImageList = new ImageList();
            myImageList.Images.Add(Resources.IconURA);
            myImageList.Images.Add(Resources.IconCentre);
            myImageList.Images.Add(Resources.IconPatient);
            myImageList.Images.Add(Resources.IconCalendar);
            myImageList.Images.Add(Resources.IconScan);

            ImageList = myImageList;
            ImageIndex = 0;
            SelectedImageIndex = 1;
            ItemHeight = 19;

            BeginUpdate();

            _rootTreeNode = new TreeNode(Resources.StrImageTreeRootNode);
            _rootTreeNode.SelectedImageIndex = _rootTreeNode.ImageIndex = 0;

            ShowRootLines = false;
            Nodes.Add(_rootTreeNode);
            _initDone = true;
            TotalImagesInTree = 0;

            EndUpdate();
        }

        public void ClearAllNodes()
        {
            _rootTreeNode.Nodes.Clear();
            _unknownCentreNode = null;
            if ( _axezDICOMX != null )
                _axezDICOMX.CtlVisible = false;
            TotalImagesInTree = 0;
        }

        public int TotalImagesInTree
        {
            get { return _totalImagesInTree; }
            set 
            {
                _totalImagesInTree = value;
                DisplayImagesListedInTree();
            }
        }

        public TreeNode RootNode { get { return _rootTreeNode; } }

        public ImageRecord SelectedImageItem
        {
            get
            {
                ImageRecord rc = null;
                DicomFile file = SelectedNode.Tag as DicomFile;
                if ((file == null) || (file.Record == null))
                    throw new UniException("OSA-061", UniException.Reset.AbortOp, UniException.Category.User, "User did not select an item containing a DicomFile or its record was null");
                rc = file.Record;
                return rc;
            }
        }

        public bool IsImageSelected()
        {
            bool rc = false;
            DicomFile file = SelectedNode.Tag as DicomFile;
            if ((file != null) && (file.Record != null))
                rc = true;
            return rc;
        }

        public void RemoveSelectedItem()
        {
            if ((SelectedNode == null) || (SelectedNode.Parent == null) || (SelectedNode.Tag as DicomFile == null))
                throw new UniException("OSA-063", UniException.Reset.AbortOp, UniException.Category.User, "No node selected, or user did not select an item containing a DicomFile or it had no parent item");

            TreeNode selected = SelectedNode;
            TreeNode next = selected.PrevNode;
            if (next == null )
                next = selected.Parent;
            selected.Parent.Nodes.Remove(selected);
            SelectedNode = next;
            RefreshAllNodes();
        }

        public TreeNode GetNode(int index)
        {
            TreeNode rc = null;
            int Count = 1;
            if (index == 0)
                rc = _rootTreeNode;
            else
            {
                foreach (TreeNode centre in _rootTreeNode.Nodes)
                {
                    if ((rc = ImageRecordTreeView.BreakOnIndexMatch(Count++, index, rc, centre)) != null)
                        break;
                    foreach (TreeNode volunteer in centre.Nodes)
                    {
                        if ((rc = ImageRecordTreeView.BreakOnIndexMatch(Count++, index, rc, volunteer)) != null)
                            break;
                        foreach (TreeNode scandate in volunteer.Nodes)
                        {
                            if ((rc = ImageRecordTreeView.BreakOnIndexMatch(Count++, index, rc, scandate)) != null)
                                break;
                            foreach (TreeNode image in scandate.Nodes)
                            {
                                if ((rc = ImageRecordTreeView.BreakOnIndexMatch(Count++, index, rc, image)) != null)
                                    break;
                            }
                        }
                    }
                }
            }
            return rc;
        }

        private static TreeNode BreakOnIndexMatch(int Count, int index, TreeNode rc, TreeNode node)
        {
            if (rc == null)
            {
                if (Count == index)
                    rc = node;
            }
            return rc;
        }

        public int PopulateTree(int VolunteerRef, string DiskRef, bool ClearExisting)
        {
            BeginUpdate();

            if (ClearExisting)
                ClearAllNodes();

            ImageRecord[] records = null;
            if ( DiskRef != null )
               records = _databases.GetAllImageDetailsForDiskRef(DiskRef);
            if (VolunteerRef != -1)
                records = _databases.GetAllImageDetailsForVolunteerRef(VolunteerRef);

            foreach (ImageRecord record in records)
                PopulateCentres(record);

            ExpandAll();

            if (VolunteerRef != -1)
                SelectTreeItem(null, null, VolunteerRef.ToString(), null);

            EndUpdate();

            return DisplayImagesListedInTree();
        }

        private void PopulateCentres(ImageRecord record)
        {
            TreeNode VolunteerCentre = null;
            foreach (TreeNode centre in _rootTreeNode.Nodes)
            {
                RegionalCentreElement hospital = centre.Tag as RegionalCentreElement;
                if (hospital != null)
                {
                    if (hospital.Number == record.VolunteerRef.Substring(0, 2))
                    {
                        VolunteerCentre = centre;
                        break;
                    }
                }
            }
            if (VolunteerCentre == null)
            {
                RegionalCentreElement hospital = GetRegionCentre(record.VolunteerRef.Substring(0, 2));
                if (hospital.Number == UnknownCentreNumber)
                {
                    if (_unknownCentreNode == null)
                    {
                        _unknownCentreNode = new TreeNode(hospital.Name);
                        _unknownCentreNode.SelectedImageIndex = _unknownCentreNode.ImageIndex = 1;
                        _unknownCentreNode.Tag = hospital;
                        _rootTreeNode.Nodes.Add(_unknownCentreNode);
                    }
                    VolunteerCentre = _unknownCentreNode;
                }
                else
                {
                    VolunteerCentre = new TreeNode(hospital.Name);
                    VolunteerCentre.SelectedImageIndex = VolunteerCentre.ImageIndex = 1;
                    VolunteerCentre.Tag = hospital;
                    _rootTreeNode.Nodes.Add(VolunteerCentre);
                }
            }
            PopulateVolunteers(record, VolunteerCentre);
        }

        private void PopulateVolunteers(ImageRecord record, TreeNode Centre)
        {
            TreeNode volunteer = null;
            foreach (TreeNode patient in Centre.Nodes)
            {
                if (patient.Text == record.VolunteerRef)
                {
                    volunteer = patient;
                    break;
                }
            }
            if (volunteer == null)
            {
                volunteer = new TreeNode(record.VolunteerRef);
                volunteer.SelectedImageIndex = volunteer.ImageIndex = 2;
                volunteer.Tag = TagVolunteerRef;
                Centre.Nodes.Add(volunteer);
            }
            PopulateScanDates(record, volunteer);
        }

        private void PopulateScanDates(ImageRecord record, TreeNode Volunteer)
        {
            TreeNode appointmentDate = null;
            foreach (TreeNode scanDate in Volunteer.Nodes)
            {
                if (scanDate.Text == record.AppointmentDate.ToString(ImageRecord.DateFormat))
                {
                    appointmentDate = scanDate;
                    break;
                }
            }
            if (appointmentDate == null)
            {
                appointmentDate = new TreeNode(record.AppointmentDate.ToString(ImageRecord.DateFormat));
                appointmentDate.SelectedImageIndex = appointmentDate.ImageIndex = 3;
                appointmentDate.Tag = TagScanDate;
                Volunteer.Nodes.Add(appointmentDate);
            }
            PopulateImages(record, appointmentDate);
        }

        private void PopulateImages(ImageRecord record, TreeNode AppointmentDate)
        {
            TreeNode pic = null;
            foreach (TreeNode image in AppointmentDate.Nodes)
            {
                DicomFile file = image.Tag as DicomFile;
                if ((file != null) && (file.Record != null) && (file.Record.ImageID == record.ImageID))
                {
                    pic = image;
                    break;
                }
            }
            if (pic == null)
            {
                pic = new TreeNode(record.GetShortDicomRef());
                pic.SelectedImageIndex = pic.ImageIndex = 4;
                pic.Tag = new DicomFile(record);
                AppointmentDate.Nodes.Add(pic);
                _totalImagesInTree++;
            }
        }

        private int DisplayImagesListedInTree()
        {
            if (_initDone)
                _statusLabelToolStrip.Text = ImagesDisplayedMessage;
            return _totalImagesInTree;
        }

        public string ImagesDisplayedMessage
        {
            get
            {
                string rc = "";
                if (_totalImagesInTree == 0)
                    rc = Resources.StrNoImagesInTree;
                else if (_totalImagesInTree == 1)
                    rc = Resources.StrOneImageInTree;
                else
                    rc = _totalImagesInTree.ToString() + " " + Resources.StrImagesInTree;
                return rc;
            }
        }

        public void SelectTreeItem(ImageRecord record, string ScanDate, string VolunteerRef, RegionalCentreElement hospital)
        {
            SelectedNode = _rootTreeNode;
            foreach (TreeNode centre in _rootTreeNode.Nodes)
            {
                if (centre.Tag == hospital)
                {
                    SelectedNode = centre;
                    break;
                }
                SelectVolunteerItemForCentre(centre, record, ScanDate, VolunteerRef); 
            }
            return;
        }
        
        private void SelectVolunteerItemForCentre(TreeNode centre, ImageRecord record, string ScanDate, string VolunteerRef) 
        {
            foreach (TreeNode volunteer in centre.Nodes)
            {
                if (volunteer.Text == VolunteerRef)
                {
                    SelectedNode = volunteer;
                    break;
                }
                SelectScanDateItemForVolunteer(volunteer, record, ScanDate); 
            }
            return;       
        }

        private void SelectScanDateItemForVolunteer(TreeNode volunteer, ImageRecord record, string ScanDate) 
        {
            foreach (TreeNode scandate in volunteer.Nodes)
            {
                if (scandate.Text == ScanDate)
                {
                    SelectedNode = scandate;
                    break;
                }
                SelectImageItemForScanDate(scandate, record); 
            }
            return;
        }

        private void SelectImageItemForScanDate(TreeNode scandate, ImageRecord record) 
        {
            foreach (TreeNode image in scandate.Nodes)
            {
                DicomFile file = image.Tag as DicomFile;
                if ((file != null) && (file.Record != null) && (record != null))
                {
                    if (file.Record.ImageID == record.ImageID)
                    {
                        SelectedNode = image;
                        break;
                    }
                }
            }
            return;
        }

        public ImageRecord[] GetDicomRecords()
        {
            List<ImageRecord> list = new List<ImageRecord>();

            foreach (TreeNode centre in _rootTreeNode.Nodes)
            {
                foreach (TreeNode volunteer in centre.Nodes)
                {
                    foreach (TreeNode scandate in volunteer.Nodes)
                    {
                        foreach (TreeNode image in scandate.Nodes)
                        {
                            DicomFile file = image.Tag as DicomFile;
                            if ((file!= null) && (file.Record != null))
                                list.Add(file.Record);
                        }
                    }
                }
            }
            ImageRecord[] arr = new ImageRecord[list.Count];
            list.CopyTo(arr);
            return arr;
        }

       

        private RegionalCentreElement GetRegionCentre(string number)
        {
            RegionalCentreElement rc = null;

            RegionalCentreElement unknownCentre = null;
            foreach (RegionalCentreElement centre in _centres.GetList())
            {
                if (centre.Number == number)
                {
                    rc = centre;
                    break;
                }
                if (centre.Number == UnknownCentreNumber)
                    unknownCentre = centre;
            }
            if (rc == null)
            {
                if (unknownCentre == null)
                    throw new UniException("OSA-041", UniException.Reset.AbortOp, UniException.Category.Resource, "unknown centre (99) not found in config file");
                else
                    rc = unknownCentre;
            }
            return rc;
        }

        public void RefreshAllNodes()
        {
            ImageRecord[] records = GetDicomRecords();

            DicomFile file = SelectedNode.Tag as DicomFile;
            RegionalCentreElement hospital = SelectedNode.Tag as RegionalCentreElement;
            string ScanDate = SelectedNode.Tag as string;
            if (ScanDate == TagScanDate)
                ScanDate = SelectedNode.Text;
            string VolunteerRef = SelectedNode.Tag as string;
            if (VolunteerRef == TagVolunteerRef)
                VolunteerRef = SelectedNode.Text;

            BeginUpdate();

            ClearAllNodes();

            foreach (ImageRecord record in records)
            {
                _databases.LoadImageDetailsForImageID(record);
                PopulateCentres(record);
            }

            ExpandAll();

            SelectTreeItem((file != null) ? file.Record : null, ScanDate, VolunteerRef, hospital);

            EndUpdate();
            DisplayImagesListedInTree();

        }

        protected override void OnAfterSelect(TreeViewEventArgs e)
        {
            base.OnAfterSelect(e);

            if (SelectedNode != null)
            {
                DicomFile file = SelectedNode.Tag as DicomFile;
                if (file == null)
                {
                    if (_axezDICOMX != null)
                        _axezDICOMX.CtlVisible = false;
                    _deleteMenu.Enabled = false;
                }
                else
                {
                    try
                    {
                        _deleteMenu.Enabled = true;
                        _statusLabelToolStrip.Text = ImagesDisplayedMessage + Resources.StrSelectedImageItem + file.ToString();
                        file.WriteToDisk(_databases, false, _tempFile);
                        if (_axezDICOMX != null)
                        {
                            _axezDICOMX.DCMfilename = _tempFile;
                            _axezDICOMX.CtlVisible = true;
                        }
                    }
                    catch (Exception ex)
                    {
                        throw new UniException("OSA-058", UniException.Reset.AbortOp, UniException.Category.System, ex, "error loading treeview selected image file {0}", file.Record.ToString());
                    }
                }
            }
        }
    }
}
