using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;
using System.IO;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for DicomFileTests
    /// </summary>
    [TestClass]
    public class DicomFileTests
    {
        private OsDatabases db;

        public DicomFileTests()
        {
            db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
        }

        [TestInitialize()]
        public void TestInitialize()
        {
        }

        [ClassInitialize()]
        public static void DicomFileTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void CtorNullParam()
        {
            DicomFile file = new DicomFile((ImageRecord)null);
        }

        [TestMethod]
        public void ConvertPatientIDBadParam()
        {
            Assert.AreEqual("-1", DicomFile.ConvertPatientID(null));
        }
        [TestMethod]
        public void ConvertStudyDateGood()
        {
            Assert.AreEqual(new DateTime(2007, 7, 25), DicomFile.ConvertStudyDate("20070725"));
        }
        [TestMethod]
        public void ConvertStudyDateNull()
        {
            Assert.AreEqual(DateTime.MinValue, DicomFile.ConvertStudyDate(null));
        }
        [TestMethod]
        public void ConvertStudyDateTooShort()
        {
            Assert.AreEqual(DateTime.MinValue, DicomFile.ConvertStudyDate("070725"));
        }
        [TestMethod]
        public void ConvertStudyDateBad()
        {
            Assert.AreEqual(DateTime.MinValue, DicomFile.ConvertStudyDate("20071325"));
        }

        [TestMethod]
        public void IsValidFilenameBad()
        {
            FileInfo fileinfo = new FileInfo("c:\\wills.txt");
            Assert.IsFalse(DicomFile.IsValidFilename(fileinfo));
        }
        [TestMethod]
        public void IsValidFilenameGood()
        {
            FileInfo fileinfo = new FileInfo("c:\\I0000001.dicom");
            Assert.IsTrue(DicomFile.IsValidFilename(fileinfo));
        }
        [TestMethod]
        public void IsValidFilenameExtZero()
        {
            FileInfo fileinfo = new FileInfo("c:\\I0000001");
            Assert.IsTrue(DicomFile.IsValidFilename(fileinfo));
        }
        [TestMethod]
        public void IsValidFilenameNull()
        {
            Assert.IsFalse(DicomFile.IsValidFilename(null));
        }

        [TestMethod]
        public void ToStringOK()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            Assert.IsTrue(file.ToString().EndsWith("Test000"));
        }

        [TestMethod]
        public void ReadFromDiskNullProgError()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            Assert.IsFalse(file.ReadFromDisk(db, null));
        }

        [TestMethod]
        public void ReadFromDiskBadFile()
        {
            DicomFile badfile = new DicomFile(ExecutionTime.Instance.InvalidSampleDicomFile);
            Assert.IsFalse(badfile.ReadFromDisk(db, MockProgram.Instance)); 
        }

        [TestMethod]
        public void ReadFromDiskException()
        {
            DicomFile badfile = new DicomFile("c:\\missing.file");
            Assert.IsFalse(badfile.ReadFromDisk(db, MockProgram.Instance));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void WriteToDiskBadParam1()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            file.WriteToDisk(null, false, ExecutionTime.Instance.DataDir); 
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void WriteToDiskBadParam3()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            file.WriteToDisk(db, false, null);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void WriteToDiskBadFile()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            Assert.IsTrue(file.ReadFromDisk(db, MockProgram.Instance)); //false if record is already in database - delete record in db and retry
            file.WriteToDisk(db, true, ExecutionTime.Instance.DataDir);
        }

        [TestMethod]
        public void CreateDicomFileName()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);

            Assert.IsTrue(file.ReadFromDisk(db, MockProgram.Instance)); //false if record is already in database - delete record in db and retry
            Assert.IsTrue(file.IsDicomHeader());
            Assert.AreEqual("99000002-2005-01-21-0000000.1.1.0" + DicomFile.StdExt, file.CreateDicomFilename());
        }

        [TestMethod]
        public void UpdateAndMaskCacheNullProgError()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);

            Assert.IsTrue(file.ReadFromDisk(db, MockProgram.Instance)); //false if record is already in database - delete record in db and retry
            Assert.IsFalse(file.UpdateAndMaskCache(null));
        }

        [TestMethod]
        public void UpdateAndMaskCacheException()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);

            Assert.IsFalse(file.UpdateAndMaskCache(MockProgram.Instance));
        }
    }
}
