using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;


namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for DicomRecordTest
    /// </summary>
    [TestClass]
    public class ImageRecordTests
    {
        private ImageRecord _rec;
        private string _testFile;

        public ImageRecordTests()
        {
            _rec = new ImageRecord();

            if (TestSuiteSetup.CheckTestImageFile000() == false)
                TestSuiteSetup.CreateTestImageFile000();
            _testFile = ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename;
        }

        [ClassInitialize()]
        public static void ImageRecordTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void FileCtor()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.AreEqual(_testFile, rec.DicomPathFilename);
        }

        [TestMethod]
        public void Details()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.IsNotNull(rec.Details);
            String temp = rec.VolunteerRef;
            rec.VolunteerRef = "";
            Assert.IsNotNull(rec.Details);
            rec.VolunteerRef = temp;
        }
        [TestMethod]
        public void GetShortDicomRefNull()
        {
            Assert.AreEqual("(null)", ImageRecord.GetShortDicomRef(null));
        }
        [TestMethod]
        public void MatchDicomRefAnyRevisionNull()
        {
            Assert.IsFalse(ImageRecord.MatchDicomRefAnyRevision(null, "999"));
            Assert.IsFalse(ImageRecord.MatchDicomRefAnyRevision("999", null));
        }
        [TestMethod]
        public void MatchDicomRefAnyRevisionNoDots()
        {
            Assert.IsFalse(ImageRecord.MatchDicomRefAnyRevision("999", "999"));
            Assert.IsFalse(ImageRecord.MatchDicomRefAnyRevision("999", "99.9"));
            Assert.IsFalse(ImageRecord.MatchDicomRefAnyRevision("99.9", "999"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("99.9", "99.9"));
        }
        [TestMethod]
        public void UpdateDicomRefNull()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.AreEqual("", rec.UpdateDicomRef(null));
        }

        [TestMethod]
        public void DicomFileSize()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.AreEqual(0, rec.DicomFileSize);
        }

        [TestMethod]
        public void FlagDetailsUnspecError()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            rec.UnspecifiedErrorText = "error";
            Assert.AreEqual("[XXX] ", rec.FlagDetails);
        }

        [TestMethod]
        public void FlagDetailsOrphan()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            rec.Orphan = true;
            Assert.AreEqual("[O--] ", rec.FlagDetails);
        }
        [TestMethod]
        public void FlagDetailsDuplicate()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            rec.Duplicate = true;
            Assert.AreEqual("[-D-] ", rec.FlagDetails);
        }
        [TestMethod]
        public void FlagDetailsMismatch()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            rec.NameMismatch = true;
            Assert.AreEqual("[--N] ", rec.FlagDetails);
        }

        [TestMethod]
        public void GetUnspecErrorText()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            rec.UnspecifiedErrorText = "error";
            Assert.AreEqual("error", rec.UnspecifiedErrorText);
        }

        [TestMethod]
        public void CheckToStringImport()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.AreEqual("[---] " + _testFile, rec.ToString());
        }

        [TestMethod]
        public void CheckToStringExport()
        {
            ImageRecord rec = new ImageRecord();
            rec.AppointmentDate = new DateTime(2006, 7, 17);
            rec.VolunteerRef = "9900002";
            Assert.AreEqual("9900002-2006-07-17", rec.ToString());
        }

        [TestMethod]
        public void IsInitiated()
        {
            ImageRecord rec = new ImageRecord(_testFile);
            Assert.IsFalse(rec.HeaderFieldsInitiated());
        }

        [TestMethod]
        public void GetErrorTextNone()
        {
            ImageRecord rec = new ImageRecord();
            Assert.AreEqual("", rec.ValidationFlagDetails);
        }

        [TestMethod]
        public void GetErrorTextAll()
        {
            ImageRecord rec = new ImageRecord();
            rec.Duplicate = true;
            rec.Orphan = true;
            rec.NameMismatch = true;
            rec.UnspecifiedErrorText = "something happened";

            string expectedText = Resources.StrErrorOrphan + Utils.StrLFCR + Resources.StrErrorDuplicate + Utils.StrLFCR + Resources.StrErrorNameMismatch + Utils.StrLFCR + Resources.StrErrorUnspec + "something happened";
            Assert.AreEqual(expectedText, rec.ValidationFlagDetails + rec.UnspecifiedErrorDetails);
        }

        [TestMethod]
        public void ErrorFlags()
        {
            ImageRecord rec = new ImageRecord();
            Assert.IsFalse(rec.Orphan);
            rec.Orphan = true;
            Assert.IsTrue(rec.Orphan);

            Assert.IsTrue(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            rec.NameMismatch = true;
            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);

            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsFalse(rec.Duplicate);
            rec.Duplicate = true;
            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);

            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsFalse(rec.UnspecifiedError);
            rec.UnspecifiedErrorText = "hello";
            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);

            rec.Orphan = false;
            Assert.IsFalse(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);

            rec.NameMismatch = false;
            Assert.IsFalse(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);

            rec.Duplicate = false;
            Assert.IsFalse(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            Assert.IsFalse(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);
        }

        [TestMethod]
        public void Flags()
        {
            ImageRecord rec = new ImageRecord();
            Assert.IsFalse(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            Assert.IsFalse(rec.Duplicate);
            Assert.IsFalse(rec.UnspecifiedError);

            byte[] flags = rec.GetFlags();
            Assert.AreEqual(10, flags.Length);
            Assert.AreEqual(0, (int)flags[0]);
            Assert.AreEqual(0, (int)flags[1]);
            Assert.AreEqual(0, (int)flags[2]);
            Assert.AreEqual(0, (int)flags[3]);

            rec.Orphan = true;
            rec.Duplicate = true;
            rec.NameMismatch = true;
            rec.UnspecifiedErrorText = "hello";
            flags = rec.GetFlags();
            Assert.AreEqual(10, flags.Length);
            Assert.AreEqual(1, (int)flags[0]);
            Assert.AreEqual(1, (int)flags[1]);
            Assert.AreEqual(1, (int)flags[2]);
            Assert.AreEqual(1, (int)flags[3]);

            flags[0] = 0;
            flags[1] = 0;
            flags[2] = 0;
            flags[3] = 0;
            rec.SetFlags(flags);
            Assert.IsFalse(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            Assert.IsFalse(rec.Duplicate);
            Assert.IsFalse(rec.UnspecifiedError);

            flags[0] = 1;
            flags[1] = 1;
            flags[2] = 1;
            flags[3] = 1;
            rec.SetFlags(flags);
            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);

        }

        [TestMethod]
        public void ResetErrorFlags()
        {
            ImageRecord rec = new ImageRecord();
            rec.Orphan = true;
            rec.Duplicate = true;
            rec.NameMismatch = true;
            rec.UnspecifiedErrorText = "hello";

            Assert.IsTrue(rec.Orphan);
            Assert.IsTrue(rec.NameMismatch);
            Assert.IsTrue(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);

            rec.ResetValidationFlags();

            Assert.IsFalse(rec.Orphan);
            Assert.IsFalse(rec.NameMismatch);
            Assert.IsFalse(rec.Duplicate);
            Assert.IsTrue(rec.UnspecifiedError);
        }

        [TestMethod]
        public void ResetErrors()
        {
            ImageRecord rec = new ImageRecord();
            Assert.IsFalse(rec.IsAnyValidationFlagSet());

            rec.Orphan = true;
            Assert.IsTrue(rec.IsAnyValidationFlagSet());
            rec.ResetValidationFlags();
            Assert.IsFalse(rec.IsAnyValidationFlagSet());

            rec.NameMismatch = true;
            Assert.IsTrue(rec.IsAnyValidationFlagSet());
            rec.ResetValidationFlags();
            Assert.IsFalse(rec.IsAnyValidationFlagSet());

            rec.Duplicate = true;
            Assert.IsTrue(rec.IsAnyValidationFlagSet());
            rec.ResetValidationFlags();
            Assert.IsFalse(rec.IsAnyValidationFlagSet());

            rec.UnspecifiedErrorText = "hello";
            Assert.IsTrue(rec.UnspecifiedError);
            Assert.IsFalse(rec.IsAnyValidationFlagSet());
        }

        [TestMethod]
        public void IsNameMatch()
        {
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan^Andy", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan^Andy^Will", "Ryan^Dndy^Will"));
            Assert.IsTrue(DicomFile.IsNameMatch("Andy^Ryan", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("A.^Ryan", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan^A", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan^^^^^Andy", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("RYAN^ANDY", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan^And", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan,^Andy", "Ryan^Andy"));
            Assert.IsTrue(DicomFile.IsNameMatch("Ryan.Andy", "Ryan^Andy"));         //BF-26
            Assert.IsTrue(DicomFile.IsNameMatch(".Andy.Ryan.", "Ryan^Andy"));       //BF-26    
            Assert.IsTrue(DicomFile.IsNameMatch("Andy. Ryan", "Ryan^Andy"));        //BF-26
            Assert.IsTrue(DicomFile.IsNameMatch("Andy .Ryan", "Ryan^Andy"));        //BF-26
            Assert.IsTrue(DicomFile.IsNameMatch(". Andy . Ryan", "Ryan^Andy"));     //BF-26
            Assert.IsTrue(DicomFile.IsNameMatch(" .Andy .Ryan", "Ryan^Andy"));      //BF-26
        }

        [TestMethod]
        public void ConvertPatientID()
        {
            Assert.AreEqual("99000001", DicomFile.ConvertPatientID("99000001U"));
            Assert.AreEqual("99000001", DicomFile.ConvertPatientID("9900001U"));
            Assert.AreEqual("99000001", DicomFile.ConvertPatientID("9900001UUUUU"));
            Assert.AreEqual("99000001", DicomFile.ConvertPatientID("9900001"));
            Assert.AreEqual("99000001", DicomFile.ConvertPatientID("9900001u"));
            Assert.AreEqual("9", DicomFile.ConvertPatientID("9U"));
            Assert.AreEqual("-1", DicomFile.ConvertPatientID("U"));
            Assert.AreEqual("-1", DicomFile.ConvertPatientID("ABC"));
        }

        [TestMethod]
        public void WriteToDisk()
        {
            ImageRecord rec = new ImageRecord();
            rec.SetDicomFileCache(new byte[100]);
            DicomFile file = new DicomFile(rec);
            string filename = Path.GetTempFileName();
            file.WriteToDisk(new OsDatabases(), false, filename);
            FileInfo info = new FileInfo(filename);
            Assert.AreEqual(100, (int)info.Length);
            File.Delete(filename);
        }

        [TestMethod]
        public void ReadFileHeader()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.DataDir + TestSuiteSetup.TestFile000Filename);
            OsDatabases db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);

            Assert.IsTrue(file.ReadFromDisk(db, MockProgram.Instance)); //false if record is already in database - delete record in db and retry
            Assert.AreEqual(TestSuiteSetup.TestFile000CranfieldDicomRef, file.Record.DicomRef);
            Assert.AreEqual(TestSuiteSetup.TestFile000PatientName, file.Record.PatientName);
            Assert.AreEqual(TestSuiteSetup.TestFile000PatientID, file.Record.VolunteerRef);
            Assert.AreEqual(TestSuiteSetup.TestFile000StudyDate, file.Record.AppointmentDate.ToString("yyyyMMdd"));
            Assert.IsTrue(file.Record.HeaderFieldsInitiated());
        }


        [TestMethod]
        public void ImageID()
        {
            _rec.ImageID = "1";
            Assert.AreEqual("1", _rec.ImageID);
        }
        [TestMethod]
        public void AppointmentDate()
        {
            DateTime tim = new DateTime(2006, 6, 21);
            _rec.AppointmentDate = tim;
            Assert.AreEqual(2006, _rec.AppointmentDate.Year);
            Assert.AreEqual(6, _rec.AppointmentDate.Month);
            Assert.AreEqual(21, _rec.AppointmentDate.Day);
        }
        [TestMethod]
        public void VolunteerRef()
        {
            _rec.VolunteerRef = "9900001";
            Assert.AreEqual("9900001", _rec.VolunteerRef);
        }
        [TestMethod]
        public void PatientName()
        {
            _rec.PatientName = "Stott^Will";
            Assert.AreEqual("Stott^Will", _rec.PatientName);
        }
        [TestMethod]
        public void DicomRef()
        {
            _rec.DicomRef = TestSuiteSetup.TestImageSonoViewDicomRef;
            Assert.AreEqual(TestSuiteSetup.TestImageDicomRef, _rec.DicomRef);
        }
        [TestMethod]
        public void ImagePathFilename()
        {
            _rec.DicomPathFilename = "image";
            Assert.AreEqual("image", _rec.DicomPathFilename);
        }
        [TestMethod]
        public void MaskPathFilename()
        {
            _rec.MaskFileLabel = "mask";
            Assert.AreEqual("mask", _rec.MaskFileLabel);
        }
        [TestMethod]
        public void ArchiveDate()
        {
            DateTime tim = new DateTime(2006, 6, 21);
            _rec.ArchiveDate = tim;
            Assert.AreEqual(2006, _rec.ArchiveDate.Year);
            Assert.AreEqual(6, _rec.ArchiveDate.Month);
            Assert.AreEqual(21, _rec.ArchiveDate.Day);
        }
        [TestMethod]
        public void ArchiveOperator()
        {
            _rec.ArchiveOperator = "Will";
            Assert.AreEqual("Will", _rec.ArchiveOperator);
        }

        [TestMethod]
        public void DiskLabel()
        {
            _rec.DiskLabel = "123";
            Assert.AreEqual("123", _rec.DiskLabel);
        }


        [TestMethod]
        public void GetCranfieldDicomRef()
        {
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", ImageRecord.GetCranfieldDicomRef("1.2.410.200001.1.9900.1.20050531132655297.1.1"));
            Assert.AreEqual("", ImageRecord.GetCranfieldDicomRef("1.2.410"));
            Assert.AreEqual("", ImageRecord.GetCranfieldDicomRef("1.2.410.200001.1.9900.1.20050531132655297.1.11.2.410.200001.1.9900.1.20050531132655297.1.1"));
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", ImageRecord.GetCranfieldDicomRef("1.2.826.999999.1.9900.1.20050531132655297.1.1.0"));
        }

        [TestMethod]
        public void SetDicomRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.0";
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", rec.DicomRef);
            rec.DicomRef = "1.2.410.200001.1.9900.1.20050531132655297.1.1";
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", rec.DicomRef);
        }

        [TestMethod]
        public void GetRevisionNumber()
        {
            ImageRecord rec = new ImageRecord();
            //OSI.ANSIBODY.COUNTRY.ORGCODE.DEVICETYPE.SERIALNO.1.STUDYDATE,STUDYHR,6or7DIGITUNIQUECODE.SERIESNO.IMAGENO.Rev
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.0";
            Assert.AreEqual(0, ImageRecord.GetRevisionNumber(rec.DicomRef));
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.1";
            Assert.AreEqual(1, ImageRecord.GetRevisionNumber(rec.DicomRef));
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.10";
            Assert.AreEqual(10, ImageRecord.GetRevisionNumber(rec.DicomRef));

            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1";
            Assert.AreEqual(-1, ImageRecord.GetRevisionNumber(rec.DicomRef));
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.";
            Assert.AreEqual(-1, ImageRecord.GetRevisionNumber(rec.DicomRef));
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.1.1";
            Assert.AreEqual(-1, ImageRecord.GetRevisionNumber(rec.DicomRef));
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.1.1.";
            Assert.AreEqual(-1, ImageRecord.GetRevisionNumber(rec.DicomRef));

        }

        [TestMethod]
        public void UpdateDicomRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "1.2.410.200001.1.9900.1.20050531132655297.1.1";
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", rec.DicomRef);
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.1", rec.UpdateDicomRef(rec.DicomRef));
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.1", rec.DicomRef);
        }

        //[TestMethod]
        //public void UpdateDicomRefBad()
        //{
        //    ImageRecord rec = new ImageRecord();
        //    rec.TestSetDicomRef("1.2.826.999999.1.9900.1.20050531132655297.1.1.A");
        //    rec.UpdateDicomRef();
        //    Assert.IsTrue(rec.UnspecifiedError);
        //    Assert.AreEqual("error", rec.GetUnspecifiedErrorDetails());
        //}

        [TestMethod]
        public void GetDicomRefMatchStr()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "1.2.826.999999.1.9900.1.20050531132655297.1.1.0";
            Assert.AreEqual("1.2.826.999999.1.9900.1.20050531132655297.1.1.%", rec.DicomRefMatchStr);

        }

        [TestMethod]
        public void MatchDicomRefAnyRevision()
        {
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", "1.2.826.999999.1.9900.1.20050531132655297.1.1.0"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.0", "1.2.826.999999.1.9900.1.20050531132655297.1.1.9"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.9", "1.2.826.999999.1.9900.1.20050531132655297.1.1.0"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.99", "1.2.826.999999.1.9900.1.20050531132655297.1.1.9"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.99", "1.2.826.999999.1.9900.1.20050531132655297.1.1.88"));
            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision("1.2.826.999999.1.9900.1.20050531132655297.1.1.1234555", "1.2.826.999999.1.9900.1.20050531132655297.1.1.1234556"));
        }
    }
}
